package actions;

import repast.simphony.random.RandomHelper;
import enums.HeuristicStrategy;

public class Heuristics 
{
	/**
	 * Returns the heuristic strategy at time of initialization or if HH type has changed to
	 * heuristic selling type. 
	 * @param calvingRate
	 * @return An Enum Strategy
	 */
	public static HeuristicStrategy getMyStrategy(int calvingRate)
	{
		HeuristicStrategy myStrategy;
		RandomHelper.createUniform(0, 1);
		double randomDraw = RandomHelper.getUniform().nextDouble();
		int strategySet = 0;//A strategy set is equal to a meta-strategy
		if(randomDraw<0.479768786)//probabilities are derived from the observed frequency of agents belonging to the strategy set
		{
			strategySet=1;
			
		}
		else if(randomDraw>=0.479768786&randomDraw<0.855491329)
		{
			strategySet=2;
		}
		else if(randomDraw>=0.855491329)
		{
			strategySet=3;
		}
		myStrategy=myPrivateStrategy(strategySet,calvingRate);
		
		return myStrategy;
	}
	
	/**
	 * Returns the initial selling age for oxen drawn from the cluster analysis
	 * . Used for initializing the binary decision array of the lcs
	 * @param myStrategy
	 * @return
	 */
	public static double getOxenStrategy(HeuristicStrategy myStrategy)
	{
		double oxenAge = 0;

		if(myStrategy!=HeuristicStrategy.AgeThree & myStrategy!=HeuristicStrategy.ClvsOne)
		{
			oxenAge=3;
		}
		else if (myStrategy==HeuristicStrategy.AgeThree)
		{
			oxenAge=4;
		}
		else if(myStrategy==HeuristicStrategy.ClvsOne)
		{
			oxenAge=3.5;
		}
		else 
		{
			throw new IllegalArgumentException ("incorect strategy");   
		}
	
		return oxenAge;
	}
	
	/**
	 * Returns the initial selling age for cows drawn from the cluster analysis
	 * . Used for initializing the binary decision array of the ga
	 * @param myStrategy
	 * @return
	 */
	public static double getCowAgeStrategy(HeuristicStrategy myStrategy)
	{
		double cowAge = 0;

		if(myStrategy==HeuristicStrategy.AgeThree||myStrategy==HeuristicStrategy.AgeFour)
		{
			cowAge=8;
		}
		else if (myStrategy==HeuristicStrategy.AgeOne)
		{
			cowAge=12;
		}
		else if (myStrategy==HeuristicStrategy.AgeTwo)
		{
			cowAge=10;
		}
		else if (myStrategy==HeuristicStrategy.AgeFive)
		{
			cowAge=6;
		}
		else if(myStrategy==HeuristicStrategy.MixOne)
		{
			cowAge=7;
		}
		else if(myStrategy==HeuristicStrategy.MixTwo)
		{
			cowAge=9;
		}
		else if(myStrategy==HeuristicStrategy.MixThree)
		{
			cowAge=11;
		}
		else
		{
			throw new IllegalArgumentException ("incorect strategy");   
		}
		
		return cowAge;
	}
	
	/**
	 * Returns the initial number of calves as a selling criteria drawn from the cluster analysis
	 * . Used for initializing the binary decision array of the lcs
	 * @param myStrategy
	 * @return
	 */
	public static double getCowCalveStrategy(HeuristicStrategy myStrategy)
	{
		double amtCalves = 0;

		if(myStrategy==HeuristicStrategy.ClvsOne || myStrategy==HeuristicStrategy.MixOne||myStrategy==HeuristicStrategy.MixThree)
		{
			amtCalves=4;
		}
		else if (myStrategy==HeuristicStrategy.ClvsTwo||myStrategy==HeuristicStrategy.MixTwo)
		{
			amtCalves=6;
		}
		else
		{
			throw new IllegalArgumentException ("incorect strategy");   
		}
		
		return amtCalves;
	}
	

	/**
	 * Helper method for the ga. Reduces the Meta-strategy from the initial Strategy. 
	 * Used to set the binary values of the initial decision array in the lcs.
	 * @param aNonMetaStrat
	 * @return
	 */
	public static HeuristicStrategy getMyMetaStrategy(HeuristicStrategy aNonMetaStrat)
	{
		HeuristicStrategy metaStrategy = null;

		if(aNonMetaStrat==HeuristicStrategy.AgeOne||aNonMetaStrat==HeuristicStrategy.AgeTwo||
				aNonMetaStrat==HeuristicStrategy.AgeThree||aNonMetaStrat==HeuristicStrategy.AgeFour||
						aNonMetaStrat==HeuristicStrategy.AgeFive)
		{
			metaStrategy=HeuristicStrategy.Age;
		}
		else if(aNonMetaStrat==HeuristicStrategy.ClvsOne||aNonMetaStrat==HeuristicStrategy.ClvsTwo)
		{
			metaStrategy=HeuristicStrategy.Calves;
		}
		else if(aNonMetaStrat==HeuristicStrategy.MixOne||aNonMetaStrat==HeuristicStrategy.MixTwo||
				aNonMetaStrat==HeuristicStrategy.MixThree)
		{
			metaStrategy=HeuristicStrategy.Mix;
		}
		else
		{
			System.out.println("no strategy!!");
		}
		
		return metaStrategy;
	}
	
	/**
	 * calculates the final strategy from the met-strategy and the calving rate. First, the
	 * calving rate determines the sub-set in the meta-strategy. Second, the observed frequency
	 * of observations in relation to the total observation in that sub-set determines the 
	 * probability of a distinct strategy. Clusters in sub-sets are derived from a hirachical 
	 * cluster analysis
	 * @param clusterSet
	 * @param calvingRt
	 * @return
	 */
	private static HeuristicStrategy myPrivateStrategy(int clusterSet, int calvingRt)
	{
		int calvingRate = calvingRt;
		if(calvingRate==18)
		{
			RandomHelper.createUniform(0,1);
			double rn=RandomHelper.getUniform().nextDouble();
			if(rn<=0.5)
			{
				calvingRate=12;
			}
			else
			{
				calvingRate=24;
			}
		}
		HeuristicStrategy myStrategy = null;
		RandomHelper.createUniform(0, 1);
		double randomNumber = RandomHelper.getUniform().nextDouble();
		if(clusterSet==1)
		{
			if(calvingRate==12)
			{
				if(randomNumber<0.092592593)
				{
					myStrategy=HeuristicStrategy.AgeFive;
				}
				else if (randomNumber>=0.092592593&randomNumber<0.31481481)
				{
					myStrategy=HeuristicStrategy.AgeTwo;
				}
				else if(randomNumber>=0.31481481)
				{
					myStrategy=HeuristicStrategy.AgeFour;
				}
			}
			else if (calvingRate==24)
			{
				if(randomNumber<0.482758621)
				{
					myStrategy=HeuristicStrategy.AgeOne;
				}
				else if (randomNumber>=0.482758621)
				{
					myStrategy=HeuristicStrategy.AgeThree;
				}
				
			}		
			else
			{
				printError();
			}
		}
		
		else if(clusterSet==2)
		{
			if(calvingRate==12)
			{			
					myStrategy=HeuristicStrategy.ClvsTwo;
			}
			else if (calvingRate==24)
			{
				
					myStrategy=HeuristicStrategy.ClvsOne;		
			}	
			else
			{
				printError();
			}
		}
		
		else if(clusterSet==3)
		{
			if(calvingRate==12)
			{
				if(randomNumber<0.380952381)
				{
					myStrategy=HeuristicStrategy.MixTwo;
				}
				else if (randomNumber>=0.380952381)
				{
					myStrategy=HeuristicStrategy.MixOne;
				}
				else
				{
					printError();
				}
				
			}
			else if (calvingRate==24)
			{
					myStrategy=HeuristicStrategy.MixThree;	
			}
			
			else
			{
				printError();
			}
		}
		return myStrategy;
	}
	
	private static void printError()
	{
		System.out.println("something wrong with the random draw in strategy calculator");
	}
}
