package actions;

import repast.simphony.random.RandomHelper;
import classifierLibrary.Situation;
import classifierLibrary.StandardSituation;
import demoZLCS.ZLCS;
import enums.HeuristicStrategy;

/**
 * Learning classifier class used with only one situation. Thus, only genetic algorithm implementation
 * @author karma
 *
 */
public class Lcs 
{
	Situation testS;
	ZLCS aClassifierSystem;
	int lastStrat;
	
	public Lcs(ZLCS ourClassifier, Situation testSituation)
	{
		aClassifierSystem=ourClassifier;
		testS=testSituation;
	}
	
	public void setNewSituation(StandardSituation sit)
	{
		this.testS=sit;
	}

	/**
	 * gets a String array with binary coded heuristic strategy wrt selling of livestock
	 * @return
	 */
	public String[] getDecisionArray()
	{		
		StandardSituation asit =  (StandardSituation) aClassifierSystem.getActionChoice(testS);
		String[] arr = new String[12];
		if(asit!=null)
		{
			for(int i=0;i<12;i++)
			{
				arr[i]=asit.getElement(i).getName();
			}
		}
		else
			System.out.println("action set is empty and thus nothing in dec array!!!!!!!!!!!!!");
			
		return arr;
	}
	
	/**
	 * processes the reward from the last action of the lcs
	 * @param reward
	 */
	public void processReward(double reward)
	{
		if(aClassifierSystem.returnActionSet()!=null)
		{	
			if(reward!=0)
			{
				aClassifierSystem.processReward(reward);
				aClassifierSystem.GA();
			}			
		}
		else
			System.out.println("action set is empty!");		
	}
	
	public double getBullSellingAge(String[] decArray)
	{
		double minBullAge=3;
		try 
			{
			  double incr = Integer.parseInt(decArray[3])+ Integer.parseInt(decArray[4])+Integer.parseInt(decArray[5]);
			  minBullAge +=(incr/2);
			} 
		catch (NumberFormatException e)
			{
			  System.out.println("String array keine Zahl!");
			}
		
		return minBullAge;
	}
	
	public double getCowSellingAge(String[] decArray)
	{
		double age=6;
		try 
			{
			  age += Integer.parseInt(decArray[6]);
			  age += Integer.parseInt(decArray[7]);
			  age += Integer.parseInt(decArray[8]);
			  age += Integer.parseInt(decArray[9]);
			  age += Integer.parseInt(decArray[10]);
			  age += Integer.parseInt(decArray[11]);
			} 
		catch (NumberFormatException e)
			{
			  System.out.println("String array keine Zahl!");
			}
		
		return age;
	}
	
	public double getCalveAmtSelling(String[] decArray)
	{
		double calves=4;
		try 
			{
			calves += Integer.parseInt(decArray[2])*2;

			} 
		catch (NumberFormatException e)
			{
			  System.out.println("String array keine Zahl!");
			}
		
		return calves;
	}
	
	public HeuristicStrategy getMyMetastrategy(String[] decArray)
	{
		HeuristicStrategy myMetaStrat =null;
		if(decArray[0].equals("0")&&decArray[1].equals("0"))
		{
			myMetaStrat= HeuristicStrategy.Age;
		}
		else if(decArray[0].equals("1")&&decArray[1].equals("0"))
		{
			myMetaStrat= HeuristicStrategy.Calves; 
		}
		else if(decArray[0].equals("0")&&decArray[1].equals("1"))
		{
			myMetaStrat= HeuristicStrategy.Mix;
		}
		else if(decArray[0].equals("1")&&decArray[1].equals("1"))
		{
			int randNr =RandomHelper.createUniform().nextIntFromTo(1, 3);
			if(randNr==1)
			{
				myMetaStrat= HeuristicStrategy.Age;
			}
			else if(randNr==2)
			{
				myMetaStrat= HeuristicStrategy.Calves; 
			}
			else if(randNr==3)
			{
				myMetaStrat= HeuristicStrategy.Mix;
			}
		}
		else
		{
			throw new IllegalArgumentException ("wrong  dec array in getMetaStrat");
		}
		
		return myMetaStrat;
	}

	public String[] getIniDecArr(HeuristicStrategy myIniStrat)
	{
		String[] arr = new String[12];
		
		HeuristicStrategy metaStrategy = Heuristics.getMyMetaStrategy(myIniStrat);
		double ageCowSale = 0;
		double ageOxenSale=0;
		double calveCowSale=0;
		
		ageOxenSale=Heuristics.getOxenStrategy(myIniStrat);
		
		if(ageOxenSale==3)
		{
			arr[3]="0";
			arr[4]="0";
			arr[5]="0";
		}
		else if(ageOxenSale==3.5)
		{
			arr[3]="1";
			arr[4]="0";
			arr[5]="0";
		}
		else if(ageOxenSale==4)
		{
			arr[3]="1";
			arr[4]="1";
			arr[5]="0";
		}
		else if(ageOxenSale==4.5)
		{
			arr[3]="1";
			arr[4]="1";
			arr[5]="1";
		}		
		else
		{
			throw new IllegalArgumentException ("ini oxen age wrong");
		}
		
		if(metaStrategy==HeuristicStrategy.Age)
		{
			ageCowSale=Heuristics.getCowAgeStrategy(myIniStrat);
			setCowsInIniDecArr(arr, Heuristics.getCowAgeStrategy(myIniStrat));
			arr[0]="0";
			arr[1]="0";
			arr[2]="0";
			
		}
		else if (metaStrategy==HeuristicStrategy.Calves)
		{
			calveCowSale=Heuristics.getCowCalveStrategy(myIniStrat);
			arr[0]="1";
			arr[1]="0";
			setCalvesIniDecArr(arr, calveCowSale);
			setIniDecArrZeros(arr, 6, 11);
		}
		else if(metaStrategy==HeuristicStrategy.Mix)
		{
			calveCowSale=Heuristics.getCowCalveStrategy(myIniStrat);
			ageCowSale=Heuristics.getCowAgeStrategy(myIniStrat);
			arr[0]="0";
			arr[1]="1";
			arr[2]="0";
			setCalvesIniDecArr(arr, calveCowSale);
			setCowsInIniDecArr(arr, ageCowSale);
		}
		else
		{
			throw new IllegalArgumentException ("wrong meta strat");
		}
		
		return arr;
	}
	
	private void setCowsInIniDecArr(String[]arr, double cowAge)
	{
		for(int z=0;z<6;z++)
		{
			arr[z+6]="0";
		}
		if(cowAge>6)
		{
			for(int i = 7; i<=cowAge;i++)
			{
				arr[i-1]="1";
			}
		}
	}
	
	private void setCalvesIniDecArr(String[]arr, double calves)
	{
		if(calves==4)
		{
			arr[2]="0";
		}
		else if(calves==6)
		{
			arr[2]="1";
		}
		else
		{
			throw new IllegalArgumentException ("ini calves amt wrong");
		}
	}
	
	private void setIniDecArrZeros(String[] arr, int start, int end)
	{
		for(int i=start;i<=end;i++)
		{
			arr[i]="0";
		}
	}		
}