package actions;

import java.util.ArrayList;
import java.util.Collections;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.util.SimUtilities;
import ses.MarketSpace;
import animals.Animal;
import animals.Bull;
import animals.Cow;
import classifierLibrary.Situation;
import classifierLibrary.StandardSituation;
import dataClasses.SellingRates;
import demoZLCS.ZLCS;
import enums.HeuristicStrategy;
import enums.SellingType;

public class SellingImpl implements Selling
{
	/**
	 * Learning classifier system - only GA used
	 */
	private Lcs myLcs;
	/**
	 * encoded decision
	 */
	private String[]decArr;
	private MarketSpace market;
	private HeuristicStrategy myMetaStrat;
	
	/**
	 * average monthly revenues for all past production cycles
	 */
	private ArrayList <Double> successAllProdCycl = new ArrayList<Double>(); 
	/**
	 * sum of monthly revenue + value of the current herd size for the the last production cycle
	 */
	private double successLastProdCycle;
	/**
	 * average monthly revenues for the last year
	 */
	private ArrayList <Double> avrMonthRevenues=  new ArrayList<Double>();
	/**
	 * real revenues over one year
	 */
	private ArrayList <Double> monthlyRevenues =  new ArrayList<Double>();
	/**
	 * current monthly revenue
	 */
	private double currAvrMonthRev;
	/**
	 * checker if production cylce has finished
	 */
	private boolean isRevsion=false;

	
	public SellingImpl(ZLCS cls, StandardSituation sit, StandardSituation secondSit, MarketSpace theMarket)
	{
		myLcs = new Lcs(cls, sit);
		decArr = myLcs.getDecisionArray();
		market=theMarket;
	}

	@Override
	public ArrayList<Animal> sellIndividual(ArrayList<Animal> allAnimals) 
	{
		ArrayList<Animal> animalsToSell = new ArrayList<Animal>();
		if(isRevsion)
		{
			myLcs.processReward(this.currAvrMonthRev+getValueOfAnimals(allAnimals));
			decArr=myLcs.getDecisionArray();
			isRevsion=false;
		}

		//get meta strategy - sell by age or # of calves or random	
		myMetaStrat = myLcs.getMyMetastrategy(decArr);
		//threshold values		
		double bullAge = myLcs.getBullSellingAge(decArr);
		double cowAge = myLcs.getCowSellingAge(decArr);
		double calves =  myLcs.getCalveAmtSelling(decArr);
		//check on consistency
		if(bullAge<3||bullAge>4.5||cowAge<6||cowAge>12||calves<4||calves>6)
			throw new IllegalArgumentException ("Animal sell parameters from lcs are wrong!!!!!");   
		
		if(myMetaStrat==HeuristicStrategy.Age)
			animalsToSell= sellAgeStrategy(bullAge, cowAge, allAnimals);
		else if(myMetaStrat==HeuristicStrategy.Calves)
			animalsToSell=sellCalveStrategy(bullAge, calves, allAnimals);
		else if(myMetaStrat==HeuristicStrategy.Mix)
			animalsToSell=sellMixStrategy(bullAge, cowAge, calves, allAnimals);
		else
			throw new IllegalArgumentException ("metastrat is null");
		
		return animalsToSell;
	}
	
	@Override
	public ArrayList<Animal> sellMaxLabourSize(ArrayList<Animal> allAnimals)
	{
		ArrayList<Animal> animalsToSell = new ArrayList<Animal>();		
		if(allAnimals.size()>=130)
			animalsToSell.addAll(sellNrOfAnimals(allAnimals,(allAnimals.size()-130)));	
		return animalsToSell;
	}
	
	public ArrayList<Animal> sellNrOfAnimals(ArrayList<Animal> allAnimals, double toSell)
	{
		ArrayList<Animal> animalsToSell = new ArrayList<Animal>();
		ArrayList<Animal> bulls = getBulls(allAnimals);
		ArrayList<Animal> cows = getCows(allAnimals);
		ArrayList<Animal> heifers = getHeifers(allAnimals);	
		
		if(toSell>0)
		{
			int count=0;
			while(!bulls.isEmpty()&&count<toSell)
			{
				Collections.sort(bulls);
				Animal aBull = bulls.get(0);
				aBull.removeMe();
				animalsToSell.add(aBull);
				bulls.remove(aBull);
				count++;
			}
			while(!cows.isEmpty()&&count<toSell)
			{
				Collections.sort(cows);
				Animal aCow = cows.get(0);
				aCow.removeMe();
				animalsToSell.add(aCow);
				cows.remove(aCow);
				count++;
			}
			while(!heifers.isEmpty()&&count<toSell)
			{
				SimUtilities.shuffle(heifers, RandomHelper.createUniform());
				Animal aHeifer = heifers.get(0);
				animalsToSell.add(aHeifer);
				aHeifer.removeMe();
				heifers.remove(aHeifer);
				count++;
			}
		}
		
		return animalsToSell;
	}

	public void storeAgriRevFromThisSale(ArrayList<Animal> animalsToSell, ArrayList<Animal>allAnimals) 
	{
		double thisRevenue = getValueOfAnimals(animalsToSell);
		monthlyRevenues.add(thisRevenue);	
		if(monthlyRevenues.size()>=12)
		{
			double sum=0;
			for(Double aRev:monthlyRevenues)
			{
				sum+=aRev;
			}
			double avrMRev = sum/monthlyRevenues.size();
			currAvrMonthRev= avrMRev;
			avrMonthRevenues.add(avrMRev);
			monthlyRevenues.clear();
			if(avrMonthRevenues.size()>=4)
			{
				double sum2=0;			
				for(Double aRev:avrMonthRevenues)
				{
					sum2+=(aRev*12);
				}
				double valueOfHerd = getValueOfAnimals(allAnimals);
				successLastProdCycle = sum2+valueOfHerd;
				isRevsion=true;
				successAllProdCycl.add(successLastProdCycle);
				avrMonthRevenues.clear();	
			}
		}
	}
	
	public void storeInvestmentCosts() 
	{
		monthlyRevenues.add(-4000.0);	
	}
	
	public double getValueOfAnimals(ArrayList<Animal> aListOfAnimals)
	{
		return market.getTotalRevenue(aListOfAnimals);
	}
			
	public void resetMyAgriRevs()
	{
		avrMonthRevenues.clear();
		currAvrMonthRev=0;
		monthlyRevenues.clear();
	}
/////helpers////////////
	
	private ArrayList<Animal> getBulls(ArrayList<Animal> allAnimals)
	{
		ArrayList<Animal> bulls = new ArrayList<Animal>();
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal instanceof Bull)
				bulls.add(anAnimal);
		}
		return bulls;
	}
	private ArrayList<Animal> getCows(ArrayList<Animal> allAnimals)
	{
		ArrayList<Animal> cows = new ArrayList<Animal>();
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal instanceof Cow)
				cows.add(anAnimal);
		}
		return cows;
	}
	
	private ArrayList<Animal> getHeifers(ArrayList<Animal> allAnimals)
	{
		ArrayList<Animal> heifers = new ArrayList<Animal>();
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal instanceof Bull)
				heifers.add(anAnimal);
		}
		return heifers;
	}
/////getters/////////////
	/**
	 * gets current average monthly agricultural profit
	 */
	public double avrMprofit()
	{
		return currAvrMonthRev;
	}
	
	/**
	 * Sell animals according to ga threhsholds + cows according their age
	 * @param bullAge
	 * @param cowSellingAge
	 * @param allAnimals
	 * @return animals to sell
	 */
	private ArrayList<Animal> sellAgeStrategy(double bullAge, double cowSellingAge, ArrayList<Animal> allAnimals) 
	{
		ArrayList<Animal> animals = new ArrayList<Animal>();
		ArrayList<Animal> soldAnimals = new ArrayList<Animal>();
		
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal instanceof Bull)
			{
				if(anAnimal.getAge()>=bullAge)
				{
					soldAnimals.add(anAnimal);
					anAnimal.removeMe();
				}
				else
					animals.add(anAnimal);					
			}
			else if(anAnimal instanceof Cow)
			{
				if(anAnimal.getAge()>cowSellingAge)
				{
					soldAnimals.add(anAnimal);
					anAnimal.removeMe();
				}
				else
					animals.add(anAnimal);					
			}
		}
		return soldAnimals;
	}
	
	/**
	 * Sell animals according to ga threhsholds + cows according the number of calves
	 * @param bullAge
	 * @param calves
	 * @param allAnimals
	 * @return
	 */
	private ArrayList<Animal> sellCalveStrategy(double bullAge, double calves, ArrayList<Animal> allAnimals) 
	{
		ArrayList<Animal> animals = new ArrayList<Animal>();
		ArrayList<Animal> soldAnimals = new ArrayList<Animal>();
		
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal instanceof Bull)
			{
				if(anAnimal.getAge()>=bullAge)
				{
					soldAnimals.add(anAnimal);
					anAnimal.removeMe();
				}
				else
					animals.add(anAnimal);					
			}
			else if(anAnimal instanceof Cow)
			{
				Cow aCow = (Cow)anAnimal;
				if(aCow.getNrOfCalves()>=calves)
				{
					soldAnimals.add(aCow);
					aCow.removeMe();
				}
				else
					animals.add(aCow);					
			}
		}
		return soldAnimals;
	}
	/**
	 * Sell animals according to ga threhsholds + what comes first: age or #of calves
	 * @param bullAge
	 * @param cowAge
	 * @param calves
	 * @param allAnimals
	 * @return
	 */
	private ArrayList<Animal> sellMixStrategy(double bullAge, double cowAge, double calves, ArrayList<Animal> allAnimals) 
	{
		ArrayList<Animal> animals = new ArrayList<Animal>();
		ArrayList<Animal> soldAnimals = new ArrayList<Animal>();
		
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal instanceof Bull)
			{
				if(anAnimal.getAge()>=bullAge)
				{
					soldAnimals.add(anAnimal);
					anAnimal.removeMe();
				}
				else
					animals.add(anAnimal);					
			}
			else if(anAnimal instanceof Cow)
			{
				Cow aCow = (Cow)anAnimal;
				if(aCow.getNrOfCalves()>=calves||aCow.getAge()>=cowAge)
				{
					soldAnimals.add(aCow);
					aCow.removeMe();
				}
				else
					animals.add(aCow);					
			}
		}
		return soldAnimals;
	}	
////////scenario method - not used in paper////////////
	@Override
	public ArrayList<Animal> sellIndividual(ArrayList<Animal> allAnimals, SellingType type, double currentMonth)
	{
		ArrayList<Animal> animalsToSell = new ArrayList<Animal>();
		if(type==SellingType.saver||type==SellingType.satisficer)
			animalsToSell= sellIndividual(allAnimals);
		else
		{
			double sellingRate = SellingRates.getSaleRateEmpDstr(type);
			animalsToSell= sellIndividualFromRate(allAnimals, sellingRate, currentMonth);		
		}
		return animalsToSell;
	}	
	
	private ArrayList<Animal> sellIndividualFromRate(ArrayList<Animal> allAnimals, double rate, double currentMonth)
	{
		ArrayList<Animal> animalsToSell = new ArrayList<Animal>();
		if(currentMonth==9)
		{
			double nrToSell = allAnimals.size()*rate;
			double nrToSellRound = Math.round(nrToSell);
			animalsToSell=sellNrOfAnimals(allAnimals, nrToSellRound);		
		}
				
		return animalsToSell;
	}
	
	public ArrayList<Animal> sellIndividualSimpleRule(ArrayList<Animal> allAnimals, double ageToSell) 
	{
		ArrayList<Animal> animalsToSell = new ArrayList<Animal>();
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal.getAge()>=ageToSell)
			{
				animalsToSell.add(anAnimal);
			}
		}
		
		for(Animal anAnimal: animalsToSell)
		{
			anAnimal.removeMe();
		}
		return animalsToSell;
	}
}
