package agents;

import java.util.ArrayList;
import java.util.HashMap;

import actions.Selling;
import actions.SellingImpl;
import animals.Animal;
import classifierLibrary.Situation;
import dataClasses.Expenditures;
import demoZLCS.ZLCS;
import enums.SellingType;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import ses.GlobSpace;

/**
 * Abstract class with implementations used by both agent types
 * @author karma
 *
 */
public abstract class AbstractHH implements SimpleHH
{
	protected Context theContext;
	protected Selling mySellingDecision;
	protected GlobSpace theGlobSpace;
	
	//HH attributes
	protected ArrayList<Animal> allAnimals=new ArrayList<Animal>();
	protected double hhSize;
	protected double age;
	protected double nonAgriInc;//total non-agricultural HH income 
	protected double agriInc;//average monthly agricultural revenues
	protected double monthSavingsMemb;//monthly savings per member
	protected double accSavingsMemb;//accumulated savings (wealth) per member
	protected double id;
	/**
	 * dynamic cooperativeness
	 */
	protected double coop; 
	/**
	 * fixed inital cooperativeness
	 */
	protected double iniCoop;
	/**
	 * expected cooperativeness
	 */
	protected double expCoop;
	/**
	 * reicprocity
	 */
	protected double recipr;
	/**
	 * expected reicprocity
	 */
	protected double expRecpr;
	/**
	 * checker for norm follower
	 */
	protected boolean cooperated;

////scenario variables////////////////
	//only used for scenarios of guest herders  not used
	protected boolean defector=false;
	//only used for allowing of flexible birth during the year not used
	protected double water;
	protected double incrBirth; 
	
	public AbstractHH(Context theContext, Selling mySellingDecision, GlobSpace theGlobSpace)
	{
		theContext.add(this);
		this.theContext=theContext;
		this.mySellingDecision =mySellingDecision;
		this.theGlobSpace=theGlobSpace;
	}

	public void iniWithFileData(HashMap<String, Double> attributes)
	{	
		//scenario parameters
		Parameters params = RunEnvironment.getInstance().getParameters();
		double coopParam = (Double)params.getValue("Cooperativeness");
		double HHattrParam = (Double)params.getValue("HH_attributes");
		
		id=attributes.get("realId");
		agriInc=attributes.get("lastAgriRev");
		
		if(HHattrParam==2)//average values for HH structure
		{
			age=54.0;
			hhSize=3;
			nonAgriInc=2204;
		}
		else if(HHattrParam==1)//full emperical heterogenity for HH structure
		{
			age=attributes.get("age");
			hhSize=attributes.get("hhSize");
			nonAgriInc=iniNonAgrIncome(attributes);
		}
		else if(HHattrParam==0)//values for HH structure drawn from a norma distribution and truncated to min max values from survey data
		{
			double rndnr = RandomHelper.createNormal(3.43, 2.131).nextDouble();
			while(rndnr<1||rndnr>13)
			{
				rndnr = RandomHelper.createNormal(3.43, 2.131).nextDouble();
			}
			hhSize = round(rndnr);
			
			rndnr = RandomHelper.createNormal(53.7, 16.391).nextDouble();
			while(rndnr<20||rndnr>94)
			{
				rndnr = RandomHelper.createNormal(53.7, 16.391).nextDouble();
			}
			age = round(rndnr);
			
			rndnr = RandomHelper.createNormal(2204, 1500).nextDouble();
			while(rndnr<150||rndnr>9360)
			{
				rndnr = RandomHelper.createNormal(2204, 1500).nextDouble();
			}
			nonAgriInc = round(rndnr);		
		}
		
		if(coopParam==1)// full empirical heterogeniy of cooperativeness.
		{
			if(attributes.get("partMeet") != null)
				iniCoop = attributes.get("partMeet")/4;
			else//Thresholds are from empirical distribution of likert scale (data) for missing data
			{
				double rndnr = RandomHelper.createUniform().nextDoubleFromTo(0, 1);
				double result=0;
				if(rndnr<0.188)
				{
					result=1;
				}
				else if(rndnr>=0.188&&rndnr<0.7125)
				{
					result=2;
				}
				else if(rndnr>=0.7125&&rndnr<0.8875)
				{
					result=3;
				}
				else 
				{
					result = 4;
				}			
				iniCoop = result/4;	
			}
		}
		else if (coopParam==0) //values for cooperativeness drawn from a normal distribution based on survey data 
		{
			double rndnr = RandomHelper.createNormal(0.55, 0.22).nextDouble();
			while(rndnr<0||rndnr>1)
			{
				rndnr = RandomHelper.createNormal(0.55, 0.22).nextDouble();
			}
			iniCoop = rndnr;
		}
								
		coop = iniCoop;
		expCoop = iniCoop;
		
		//reciprocity has no fixed part which means that it will fast adapt
		if(attributes.get("percNormSanc") != null)
			recipr = attributes.get("percNormSanc")/4;
		else // for missing data
			recipr = (RandomHelper.createUniform().nextIntFromTo(0, 4))/4;
		expRecpr = recipr;
	}
	
	public void iniWithOtherHHData(HashMap<String, Double> attributes) 
	{
		id=attributes.get("realId");
		age=attributes.get("age");
		hhSize=attributes.get("hhSize");
		agriInc=attributes.get("agriInc");
		nonAgriInc=attributes.get("nonAgriInc");
		monthSavingsMemb=attributes.get("monthSavingsMemb");
		accSavingsMemb=attributes.get("accSavingsMemb");
		coop=attributes.get("coop");
		iniCoop=attributes.get("iniCoop");
		expCoop=attributes.get("expCoop");
		recipr=attributes.get("expCoop");
		expRecpr=attributes.get("expRecpr");
		water=attributes.get("waterInKraal");
		incrBirth=attributes.get("incrBirth");		
	}

	/**
	 * initializes non-agricultural income by adding different income sources from input file
	 * @param attributes
	 * @return
	 */
	private double iniNonAgrIncome(HashMap<String, Double> attributes)
	{
		double wageInc = attributes.get("wageInc");
		double remtitt = attributes.get("privTrans");
		double oldAgeG = attributes.get("oldAgeP");
		double childG = attributes.get("childGrant");
		double orphG = attributes.get("orphanseAss");
		double disabG = attributes.get("disabGrant");
		double otherInc = attributes.get("otherInc");
		
		return wageInc+remtitt+oldAgeG+childG+orphG+disabG+otherInc;
	}

	/**
	 * returns monthly HH savings per member
	 * @return
	 */
	public double getSavings() 
	{
		return monthSavingsMemb;
	}
	
	/**
	 * returns accumulated savings per HH member
	 * @return
	 */
	public double getAccSavingsMemb()
	{
		return accSavingsMemb;
	}
		
	@Override
	public boolean isCooperator() 
	{
		return cooperated;
	}

	@Override
	public void setCooperator(boolean isCooperator) 
	{
		cooperated=isCooperator;
	}
	
	@Override
	public void iniCooperator()
	{
		double rndNr = RandomHelper.getUniform().nextDoubleFromTo(0, 1);
        if(rndNr>=0.5)
        	cooperated=true;
        else
        	cooperated=false;
	}
	
	public double getIniCoop()
	{
		return iniCoop;
	}
	
	@Override
	public void hhBalanceSimple() 
	{
		double incPerMember = (agriInc+nonAgriInc)/hhSize;
		monthSavingsMemb = incPerMember-Expenditures.getExpenditures((nonAgriInc/hhSize), (agriInc/hhSize), hhSize);
		accSavingsMemb += monthSavingsMemb;
	}
	
	/**
	 * computes HH savings for non-livestock owning HH based on selling types.
	 */
	public void hhBalanceTypes() 
	{
		SellingType type = getMySellingType();
		double incomePerMemb = (nonAgriInc+agriInc)/hhSize;
		double spendRate = Expenditures.getSpendRate(type);
		double minExp = Expenditures.getMinFoodExp()+Expenditures.getMinNonFoodExp();
		double restInc = incomePerMemb-minExp;
		double restExp = Math.max(0, (restInc*spendRate));
		double totExp = minExp+restExp;
		monthSavingsMemb = incomePerMemb-totExp;
		accSavingsMemb += monthSavingsMemb;
	}
	
	/**
	 * gets a selling type. Derived from selling rates and income (cluster analysis)
	 * @return
	 */
	protected SellingType getMySellingType()
	{
		SellingType type=null;
		if(((nonAgriInc+agriInc)/hhSize)<1000)
		{
			if (monthSavingsMemb>=150)
			{
				type=SellingType.saver;
			}
			else if(monthSavingsMemb<150)
			{
				if(((nonAgriInc+agriInc)/hhSize)<=605)
					type=SellingType.survivor;
				else
					type=SellingType.spender;
			}
		}
		else
			type=SellingType.satisficer;

		return type;
	}
		
	/**
	 * resets accumulated savings
	 */
	public void resetAccSavings()
	{
		accSavingsMemb=0;
	}
	
	/**
	 * helper method for rounding to integers
	 * @param d
	 * @return
	 */
	protected int round(double d)
	{
	    double dAbs = Math.abs(d);
	    int i = (int) dAbs;
	    double result = dAbs - (double) i;
	    if(result<0.5){
	        return d<0 ? -i : i;            
	    }else{
	        return d<0 ? -(i+1) : i+1;          
	    }
	}
	
///////////not used for current version////////////////	
	@Override
	public void setDefector(boolean isDefector) 
	{
		defector=isDefector;	
	}
	
	@Override
	public boolean isDefector() 
	{
		return defector;
	}
}
