package agents;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import actions.Selling;
import animals.Animal;
import animals.Bull;
import animals.Cow;
import animals.Heifer;
import classifierLibrary.Situation;
import classifierLibrary.StandardSituation;
import dataClasses.BirthCalculator;
import dataClasses.InputCostsCalculator;
import demoZLCS.ZLCS;
import enums.CalvingInterval;
import enums.SellingType;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import ses.GlobSpace;
import ses.LiveWeightGain;

/**
 * a livestock producing HH agent
 * @author karma
 *
 */
public class ProducerHHImpl extends AbstractHH implements ProducerHH
{
	/**
	 * number cows owned 12 months ago
	 */
	private double cowsLastYear;


	public ProducerHHImpl(Context theContext, Selling mySellingDecision, GlobSpace theGlobSpace)
	{
		super(theContext, mySellingDecision, theGlobSpace);	
	}
	
	@Override
	public void iniWithFileData(HashMap<String, Double> attributes) 
	{
		super.iniWithFileData(attributes);	
		Parameters params = RunEnvironment.getInstance().getParameters();
		//parameter of intial herd size heterogenity
		double Ini_HerdSize = (Double)params.getValue("Ini_HerdSize");
		
		double cows=0; 
		double bulls=0;
		double heifers=0;
		
		if(Ini_HerdSize==1)//full empirical heterogenity of herd sizes
		{
			cows = attributes.get("cowsIni");
			bulls = attributes.get("oxenIni");
			heifers = attributes.get("calvesIni");
		}
		else if(Ini_HerdSize==2)//average values for initial herd sizes
		{
			 cows = 3;
			 bulls = 0;
			 heifers = 2;		 
		}
		else if(Ini_HerdSize==0)//intial herd sizes drawn from normal distribution from survey data
		{
			double rndnr = RandomHelper.createNormal(3.05, 2.81).nextDouble();
			while(rndnr<1||rndnr>15)
			{
				rndnr = RandomHelper.createNormal(3.05, 2.81).nextDouble();
			}
			cows = round(rndnr);
			
			rndnr = RandomHelper.createNormal(2.075, 1.74134).nextDouble();
			while(rndnr<0||rndnr>7)
			{
				rndnr = RandomHelper.createNormal(2.075, 1.74134).nextDouble();
			}
			heifers = round(rndnr);
			
			rndnr = RandomHelper.createNormal(0.375, 0.7).nextDouble();
			while(rndnr<0||rndnr>3)
			{
				rndnr = RandomHelper.createNormal(0.375, 0.7).nextDouble();
			}
			bulls = round(rndnr);
		}
		//other HH attributes
		double savings = attributes.get("savingsMemb");
		double waterInKraal =  attributes.get("waterInKraal");
		double incrBirthRate =  attributes.get("incrBirth");
		water=waterInKraal;
		incrBirth=incrBirthRate;
		monthSavingsMemb=savings;
		cowsLastYear=cows;
		
		//initialize animals
		RandomHelper.createUniform();
		int iniAge;
		for(int i=0; i<cows;i++)
		{
			Cow aCow = new Cow(350, theContext);
			iniAge = RandomHelper.nextIntFromTo(2, 8);
			aCow.setIniAge(iniAge);
			theContext.add(aCow);
			allAnimals.add(aCow);
		}
		
		for(int i=0; i<bulls;i++)
		{
			Bull aBull = new Bull(350, theContext);
			iniAge = RandomHelper.nextIntFromTo(2,8);
			aBull.setIniAge(iniAge);
			theContext.add(aBull);
			allAnimals.add(aBull);
		}
		
		for(int i=0; i<heifers;i++)
		{
			Heifer aHeifer = new Heifer(100, theContext);
			iniAge = RandomHelper.nextIntFromTo(1, 2);
			aHeifer.setIniAge(iniAge);
			theContext.add(aHeifer);
			allAnimals.add(aHeifer);
		}	
	}

	@Override
	public void informLwg(double lwg) 
	{
		for(Animal aCattle: allAnimals)	
		{
			aCattle.incrBodyWeight(lwg);
		}		
	}
	
	public void ageYourAnimals()
	{
		ArrayList<Animal> newAdults = new ArrayList<Animal>();
		ArrayList<Animal> grownUps = new ArrayList<Animal>();
		for(Animal aCattle: allAnimals)
		{
			aCattle.aging();
			if(aCattle instanceof Heifer)
			{
				if(aCattle.getAge()>=3)
				{
					RandomHelper.createUniform();
					double randomNr = RandomHelper.nextDoubleFromTo(0, 1.0);
					if(randomNr>=0.5)
					{
						Bull aBull = new Bull(aCattle.getBodyWeight(), theContext);
						theContext.add(aBull);
						aBull.setIniAge(aCattle.getAge());
						newAdults.add(aBull);
						grownUps.add(aCattle);
					}
					else if(randomNr<0.5)
					{
						Cow aCow = new Cow(aCattle.getBodyWeight(), theContext);
						theContext.add(aCow);
						aCow.setIniAge(aCattle.getAge());
						//aCow.setCalvingInterval(calcCalvingInterval());
						newAdults.add(aCow);
						grownUps.add(aCattle);
					}
				}
			}
		}
		allAnimals.addAll(newAdults);
		for(Animal aHeifer: grownUps)
		{
			aHeifer.removeMe();
		}
		allAnimals.removeAll(grownUps);
	}
	
	@Override
	public void birthYourAnimalsBrand(double yearlyLwg) 
	{
		ArrayList<Animal> newCalves = new ArrayList<Animal>();
		ArrayList<Cow> cows = getCows();
		double brandRate = LiveWeightGain.getBrandRate(yearlyLwg);
		RandomHelper.createUniform();
		for(int i=0;i<cowsLastYear;i++)
		{
			double randNr = RandomHelper.nextDoubleFromTo(0, 1.0);
			if(randNr<=brandRate)
			{
				Heifer aHeifer = new Heifer(yearlyLwg, theContext);
				aHeifer.setIniAge(1.0);
				theContext.add(aHeifer);				
				newCalves.add(aHeifer);
			}
		}
		//assumes that all cows gave birth but not all calves survived the first year (branding rate)
		if(cowsLastYear>=cows.size())
		{
			for(Cow aCow: cows)
			{
				aCow.incNrOfCalves();
			}
		}
		else
		{
			for(int i=0;i<cowsLastYear;i++)
			{
				cows.get(i).incNrOfCalves();
			}
		}		
		allAnimals.addAll(newCalves);
	}
	
	public void dieYourAnimals(double yearlyLwg)
	{
		double mort = LiveWeightGain.getMortality(yearlyLwg);
		RandomHelper.createUniform();
		ArrayList<Animal> survivor = new ArrayList<Animal>();
		for(Animal anAnimal: allAnimals)
		{
			if(anAnimal.getAge()>1)
			{
				double randNr = RandomHelper.getUniform().nextDoubleFromTo(0, 1.0);
				if(randNr<=mort||anAnimal.getAge()>=13)
				{
					anAnimal.removeMe();
				}
				else
					survivor.add(anAnimal);
			}
			else
				survivor.add(anAnimal);
		}
		allAnimals = survivor;		
		cowsLastYear=getCows().size();
	}
	
	@Override
	public   ArrayList<Animal> sellIndividual() 
	{
		ArrayList<Animal> soldAnimals = mySellingDecision.sellIndividual(allAnimals);
		allAnimals.removeAll(soldAnimals);
		ArrayList<Animal> soldAnimalsMaxLabour = mySellingDecision.sellMaxLabourSize(allAnimals);
		allAnimals.removeAll(soldAnimalsMaxLabour);
		soldAnimals.addAll(soldAnimalsMaxLabour);	
		mySellingDecision.storeAgriRevFromThisSale(soldAnimals, allAnimals);
		
		return soldAnimals;
	}
	
	public  ArrayList<Animal>  sellNorm() 
	{
		ArrayList<Animal> soldAnimals = new ArrayList<Animal>();
		Parameters params = RunEnvironment.getInstance().getParameters();
		double maxSize =(Double) params.getValue("maxHerdSize");//maximum herd size prescribed by the norm
		
		if(cooperated&&allAnimals.size()>maxSize)
		{
			double toSell = allAnimals.size()-maxSize;
			soldAnimals.addAll(mySellingDecision.sellNrOfAnimals(allAnimals,toSell));	
		}			
		allAnimals.removeAll(soldAnimals);
		mySellingDecision.storeAgriRevFromThisSale(soldAnimals, allAnimals);
		
		return soldAnimals;
	}
	
	@Override
	public boolean isExit() 
	{
		if(allAnimals.isEmpty())
			return true;
		else
			return false;
	}

	@Override
	public EntryAble exit() 
	{
		HashMap<java.lang.String, Double> attributes = new HashMap<java.lang.String, Double>();
		EntryAble otherHH = new OtherHHImpl(theContext, this.mySellingDecision, this.theGlobSpace);
		
		otherHH.setDefector(defector);		
		attributes.put("realId",new Double (id));
		attributes.put("hhSize",new Double (hhSize));
		attributes.put("age",new Double (age));
		attributes.put("nonAgriInc",new Double (nonAgriInc));
		attributes.put("agriInc",new Double (0));
		attributes.put("monthSavingsMemb",new Double (monthSavingsMemb));
		attributes.put("accSavingsMemb",new Double (accSavingsMemb));
		attributes.put("coop",new Double (coop));
		attributes.put("iniCoop",new Double (iniCoop));
		attributes.put("expCoop",new Double (expCoop));
		attributes.put("recipr",new Double (recipr));
		attributes.put("expRecpr",new Double (expRecpr));
		attributes.put("waterInKraal",new Double (water));
		attributes.put("incrBirth",new Double (incrBirth));
		mySellingDecision.resetMyAgriRevs();

		otherHH.iniWithOtherHHData(attributes);
		otherHH.setCooperator(cooperated);	
		
		theContext.remove(this);
	
		return otherHH;
	}

	public void hhBalanceSimple()
	{
		agriInc=mySellingDecision.avrMprofit();
		super.hhBalanceSimple();
	}
	
	@Override
	public void updateCooperation(double groupRepuation, double severity) 
	{
		recipr=(expRecpr+groupRepuation)/2;
		expRecpr=groupRepuation;	
		expCoop=(1-(severity));
		coop=iniCoop*expCoop;		
	}

	@Override
	public double getConditionalCooperation() 
	{
		return (recipr+coop)/2;
	}
	
	@Override
	public double getValueOfHerd() 
	{
		return mySellingDecision.getValueOfAnimals(allAnimals);		
	}
	
//helpers 
	private ArrayList<Cow> getCows()
	{
		ArrayList<Cow> cows = new ArrayList<Cow>();
		for(Animal aCattle:allAnimals)
		{
			if(aCattle instanceof Cow)
			{
				cows.add((Cow) aCattle);
			}
		}
		return cows;
	}
	/**
	 * creates a new cow for entries
	 */
	public void addCow()
	{
		Cow aCow = new Cow(350, theContext);
		aCow.setIniAge(3);		
		theContext.add(aCow);
		allAnimals.add(aCow);
	}
	
	/**
	 * gets the tropical livestock units based on body-weight
	 * @return
	 */
	private double getMyTLU()
	{
		double sum=0;
		for(Animal anAnimal: allAnimals)
		{
			sum+=anAnimal.getBodyWeight();
		}		
		return sum/250;
	}
	
	public void storeInvestmentCosts()
	{
		mySellingDecision.storeInvestmentCosts();
	}

//accessors 
	public double getId()
	{
		return id;
	}
	
	@Override
	public Collection<? extends Animal> getAnimals() 
	{
		return allAnimals;
	}
	public double getHerdSize()
	{
		return allAnimals.size();
	}
	
	public double getAgriRev()
	{
		return agriInc;
	}
	
	public double getNonAgriIncMemb()
	{
		return nonAgriInc/hhSize;
	}
	
//not used in the current version
	

	
	public int getSellingType()
	{
		SellingType type=  getMySellingType();
		int typNr=0;
		if(type==SellingType.survivor)
		{
			typNr=1;
		}
		if(type==SellingType.spender)
		{
			typNr=2;
		}
		if(type==SellingType.saver)
		{
			typNr=3;
		}
		if(type==SellingType.satisficer)
		{
			typNr=4;
		}
		return typNr;
	}
}
