package animals;

import repast.simphony.context.Context;

/**
 * Super class for all animals
 * @author karma
 *
 */
public class Animal implements Comparable<Animal>
{	
	protected double bodyWeight;
	protected double age;
	protected Context ourContext;
	
	/**
	 * give Repast context to the animal
	 * @param theContext
	 */
	public Animal(Context theContext)
	{
		ourContext=theContext;
	}
	
	/**
	 * return the body-weight in kg
	 * @return
	 */
	public double getBodyWeight()
	{
		return bodyWeight;
	}
	
	/**
	 * increase or decrease the body weight by monthly live-weight gain or loss
	 * @param lwg
	 */
	public void incrBodyWeight(double lwg)
	{
		bodyWeight=bodyWeight+lwg;
		checkBodyWeight();	
	}
	
	/**
	 * age the animal by one month
	 */
	public void aging()
	{
		age+=(1.0/12.0);
	}
	
	/**
	 * set the initial age when born or bought
	 * @param iniAge
	 */
	public void setIniAge(double iniAge)
	{
		age=iniAge;
	}
	
	/**
	 * returns the age of the animal in months
	 * @return
	 */
	public double getAge()
	{
		return age;
	}
	
	/**
	 * removes the animal from the repast context
	 */
	public void removeMe()
	{
		ourContext.remove(this);
	}

	/**
	 * logical constrain on body-weight with a max cap of 450 kg
	 */
	private void checkBodyWeight() 
	{
		if(bodyWeight>=450)
			bodyWeight=450;
		else if(bodyWeight<=0)
			bodyWeight=0;	
	}

	/**
	 * compares the age of one animal to another animal 
	 */
	public int compareTo(Animal anotherAnimal)
	{
		if (getAge()<anotherAnimal.getAge())
			return 1;
		else if(getAge()==anotherAnimal.getAge())
			return 0;
		else
			return -1;		
	}
}
