package classifierLibrary;

/**
 * <p>Library classifierLibrary, Klasse StandardClassifier</p><p> Copyright
 * by Klaus Hufschlag 2004,2005,2006</p><p>This file is part of the Generic
 * Classifier Library. The Generic Classifier Library is free software; you can
 * redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. The Generic Classifier
 * Library is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with
 * the Generic Classifier Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.</p>
 * @author Klaus Hufschlag
 * @version 1.4
 */
public class StandardClassifier extends Classifier {
	private double strength = 0;

	/**
	 * Constructor
	 * @param r Rule for the Classifier
	 * @param strength Value for strength
	 */
	public StandardClassifier(Rule r, double strength) {
		super(r);
		setStrength(strength);
	};

	/**
	 * Sets the classifier's strength-attribute
	 * @param value Value for strength to be set
	 */
	public void setStrength(double value) {
		strength = value;
	}

	/**
	 * Returns the classifier's strength-attribute
	 * @return Strength value as double
	 */
	public double getStrength() {
		return strength;
	}

	/**
	 * Crossover-Method, extending the inherited crossover-method, setting the
	 * strenght value of the resulting classifier to the average of the previous
	 * ones.
	 * @param c StandardClassifier to cross with
	 * @param Position Position for crossover
	 * @return new StandardClassifier as result of crossover
	 * @see classifierLibrary.Classifier#cross(classifierLibrary.Classifier,int)
	 */
	public StandardClassifier cross(StandardClassifier c, int Position) {
		StandardClassifier n = (StandardClassifier) super.cross(c, Position);
		n.setStrength((this.getStrength() + c.getStrength()) / 2);
		return n;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Classifier#callFunction(java.lang.String)
	 */
	@Override
	public double callFunction(String which) {
		// just a shortcut for better performance, avoiding dynamic function
		// call:
		if (which.equals("getStrength"))
			return getStrength();
		else
			return super.callFunction(which);
	}
}