package classifierLibrary;

import java.util.*;
import simAuxLibrary.*;

/**
 * <p>Library classifierLibrary, Klasse StandardSituation</p><p>Copyright by
 * Klaus Hufschlag 2004, 2005, 2006</p><p>This file is part of the Generic
 * Classifier Library. The Generic Classifier Library is free software; you can
 * redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. The Generic Classifier
 * Library is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with
 * the Generic Classifier Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.</p>
 * @author Klaus Hufschlag
 * @version 1.4
 */
public class StandardSituation implements Situation, Cloneable {
	private Symbol[] symbols;
	private int length;
	private Language lang;

	/**
	 * Constructor, creates a string-like sequence of Symbols, with given
	 * language and lenght, initializes all places in sequence with the default
	 * symbol of the language
	 * @param l Language to use
	 * @param Length Length of the sequence
	 */
	public StandardSituation(Language l, int Length) {
		length = Length;
		symbols = new Symbol[length];
		lang = l;
		for (int lauf = 0; lauf < length; lauf++) {
			setElement(lauf, l.getDefault());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() {
		StandardSituation s = null;
		try {
			s = (StandardSituation) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		s.symbols = symbols.clone();
		return s;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#getLength()
	 */
	@Override
	public int getLength() {
		return length;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#size()
	 */
	@Override
	public int size() {
		return length;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#getLanguage()
	 */
	@Override
	public Language getLanguage() {
		return lang;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#setLanguage(classifierLibrary.Language)
	 */
	@Override
	public void setLanguage(Language lang) {
		this.lang = lang;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#compatibility(classifierLibrary.Situation)
	 */
	@Override
	public boolean compatibility(Situation sit) {
		return (this.getLength() == sit.getLength())
				&& this.getLanguage().contains(sit.getLanguage());
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#cross(classifierLibrary.Situation, int)
	 */
	@Override
	public Situation cross(Situation sit, int Position) {
		/*
		 * if (compatibility(sit) == false) { throw new
		 * SimLibraryError("Situations incompatible"); }
		 */
		StandardSituation s = (StandardSituation) sit;
		StandardSituation c = (StandardSituation) clone();
		for (int lauf = Position; lauf < length; lauf++) {
			c.setElement(lauf, s.getElement(lauf));
		}
		return c;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#mutation(double)
	 */
	@Override
	public Situation mutation(double Probability) {
		Random r = new Random();
		StandardSituation c = new StandardSituation(lang, getLength());
		Symbol s = null;
		for (int lauf = 0; lauf < length; lauf++) {
			if (r.nextDouble() < Probability) {
				s = lang.randomSymbol();
				while (getElement(lauf) == s) {
					s = lang.randomSymbol();
				}
				c.setElement(lauf, lang.randomSymbol());
			} else {
				c.setElement(lauf, getElement(lauf));
			}
		}
		return c;
	}

	/**
	 * Sets the Element of the Situation/sequence at given position to the given
	 * Symbol.
	 * @param Position Position at which the Symbol has to be set
	 * @param Symbol Symbol to set
	 */
	public void setElement(int Position, Symbol Symbol) {
		if ((0 <= Position) && (Position < length)) {
			symbols[Position] = Symbol;
		}
	}

	/**
	 * Returns the Symbol at a given position in the Situation/sequence.
	 * @param position Position of wanted Symbol
	 * @return Symbol Symbol at wanted position, null if position does not exist
	 */
	public Symbol getElement(int position) {
		Symbol s = null;
		if ((0 <= position) && (position < length)) {
			s = symbols[position];
		}
		return s;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#randomizeSituation()
	 */
	@Override
	public void randomizeSituation() {
		for (int lauf = 0; lauf < length; lauf++) {
			this.setElement(lauf, lang.randomSymbol());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#randomizeSituation(double)
	 */
	@Override
	public void randomizeSituation(double wildprob) {
		Symbol s;
		Random r = new Random();
		r.setSeed(4);
		for (int lauf = 0; lauf < length; lauf++) {
			if ((r.nextDouble() > wildprob) && lang.hasWildcards()) {
				s = lang.randomSymbol();
				while (s.hasMembers() == false) {
					s = lang.randomSymbol();
				}
			} else {
				s = lang.randomSymbol();
				while (s.hasMembers() == false) {
					s = lang.randomSymbol();
				}
			}
			this.setElement(lauf, s);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#debug_out()
	 */
	@Override
	public void debug_out() {
		DebugOutput.print("StandardSituation: ");
		for (int lauf = 0; lauf < symbols.length; lauf++) {
			if (lauf > 0) {
				DebugOutput.print(",");
			}
			DebugOutput.print(this.getElement(lauf).getName());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#compare(classifierLibrary.Situation)
	 */
	@Override
	public int compare(Situation sit) {
		int count = 0;
		StandardSituation s = (StandardSituation) sit;
		for (int lauf = 0; lauf < getLength(); lauf++) {
			if (getElement(lauf).equals(s.getElement(lauf))) {
				count++;
			}
		}
		return count;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#compareSubsumtive(classifierLibrary.Situation)
	 */
	@Override
	public int compareSubsumtive(Situation sit) {
		StandardSituation s = (StandardSituation) sit;
		int count = 0;
		for (int lauf = 0; lauf < getLength(); lauf++) {
			if (s.getElement(lauf) == null)
				throw new SimLibraryError(
						"Situation to compare contains NULL Symbol");
			if (getElement(lauf).subsumes(s.getElement(lauf))
					|| getElement(lauf).subsumes(this.getLanguage(),
							s.getElement(lauf))) {
				count++;
			}
		}
		return count;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#isMatched(classifierLibrary.Situation)
	 */
	@Override
	public boolean isMatched(Situation sit) {
		if (compare(sit) == size())
			return true;
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#isMatchedSubsumtive(classifierLibrary.Situation)
	 */
	@Override
	public boolean isMatchedSubsumtive(Situation sit) {
		if (compareSubsumtive(sit) == size())
			return true;
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see classifierLibrary.Situation#specificMembers()
	 */
	@Override
	public int specificMembers() {
		int count = 0;
		for (int lauf = 0; lauf < getLength(); lauf++) {
			if (getElement(lauf).hasMembers() == false) {
				count++;
			}
		}
		return count;
	}
}