package dataClasses;

import enums.CalvingInterval;
import repast.simphony.random.RandomHelper;

/**
 * Computes calving intervals with an orderd logit regression depending on HH characteristics. Only used for scenarios with a flexible calving rate (not used in the current paper)
 * @author karma
 *
 */
public class BirthCalculator
{
	private final double expInputCoeff = -0.0150840033507803;
	private final double ageCoeff= 0.010437535374855;
	private final double waterCoeff = -0.930157046367413;
	private final double incrBirthCoeff =-1.26212903863624;	
	private final double limit1 = -0.822646713953728;
	private final double limit2 = -0.4945126893867538;
	
	//independet variables
	private double expInp;
	private double age;
	private double water;
	private double incrBirth;
	
	public BirthCalculator(double expInp, double age2, double water2, double incrBirth2)
	{
		this.expInp=expInp;
		this.age=age2;
		this.water=water2;
		this.incrBirth=incrBirth2;
	}
	
	public CalvingInterval getCalvingInterval()
	{
		CalvingInterval theInterval;
		RandomHelper.createUniform(0,1);
		double randX = RandomHelper.getUniform().nextDouble();
		double twMonth = calcProb12Months();
		double eightMonth = calcProb18Months();
		double twfourMonth = calcProb24Months();
		
		if(randX<twMonth)
			return CalvingInterval.twelveMonth;
		else if (randX>=twMonth&&randX<eightMonth+twMonth)
			return CalvingInterval.eighteenMonths;
		else if (randX>=eightMonth+twMonth)
			return CalvingInterval.twentyFourMonths;
		else
			return null;
	}
	
	
	private double calcSumOfWeightedVar()
	{
		double expInpVar = this.expInp*this.expInputCoeff;
		double ageVar = this.age*this.ageCoeff;
		double waterVar = this.water*this.waterCoeff;
		double incrBirthVar = this.incrBirth*this.incrBirthCoeff;	
		return expInpVar+ageVar+waterVar+incrBirthVar;
	}
	
	private double calcProb12Months()
	{
		double x = this.limit1-calcSumOfWeightedVar();
		RandomHelper.createNormal(0, 1);
		double prob12Months = RandomHelper.getNormal().cdf(x);
		return prob12Months;
	}
	
	private double calcProb18Months()
	{
		double x = this.limit2-calcSumOfWeightedVar();
		double y = this.limit1-calcSumOfWeightedVar();
		RandomHelper.createNormal(0, 1);
		double prob18Months = RandomHelper.getNormal().cdf(x)-RandomHelper.getNormal().cdf(y);
		return prob18Months;
	}
	
	private double calcProb24Months()
	{
		double x = this.limit2-calcSumOfWeightedVar();
		RandomHelper.createNormal(0, 1);
		double prob24Months = 1-RandomHelper.getNormal().cdf(x);
		return prob24Months;
	}
		
	//Accessors:
	/**
	 * @param expInp the expInp to set
	 */
	public void setExpInp(double expInp) 
	{
		this.expInp = expInp;
	}

	/**
	 * @param age the age to set
	 */
	public void setAge(int age) 
	{
		this.age = age;
	}

	/**
	 * @param water the water to set
	 */
	public void setWater(int water) 
	{
		this.water = water;
	}

	/**
	 * @param incrBirth the incrBirth to set
	 */
	public void setIncrBirth(int incrBirth) 
	{
		this.incrBirth = incrBirth;
	}
}
