package dataClasses;

/**
 * logistic reggression on the probablitiy to enter livestock production
 * @author karma
 *
 */
public class Entry 
{
	private final static double constant = -5.658213;
	private final static double incMembCoeff = 0.004343;
	private final static double incMembSqrCoeff = -0.00000141;
	private final static double hhSizeCoeff = 0.482576;
	private final static double ageCoeff =0.027886;
 
	
	/**
	 * Calculates the probability of a non-livestock owning HH to enter into livestock production with a logistic regression
	 * @param incMemb
	 * @param hhSize
	 * @param age of HH head
	 * @return gives the probability based on a logit model
	 */
	public static double entryProb(double incMemb, double hhSize, double age)
	{
		double regr = constant+incMembCoeff*incMemb+incMembSqrCoeff*(Math.pow(incMemb, 2))+ageCoeff*age+hhSizeCoeff*hhSize;
		double regrPwrE = Math.exp(-1*regr);
		double logitProb = 1/(1+regrPwrE);
		
		return logitProb;
	}
}
