package dataClasses;

import java.util.ArrayList;

import enums.SellingType;
import repast.simphony.random.RandomHelper;

/**
 * OLS regression on HH expenditures
 * @author karma
 *
 */
public class Expenditures 
{
	private static final double minFoodExp = 211.1661;
	private static final double minNonFoodExp = 191.3374;
	
	/**
	 * Regression on HH expenditures from HH data
	 * @param nonAgrIncMemb
	 * @param agrIncMemb
	 * @param hhSize
	 * @return
	 */
	public static double getExpenditures(double nonAgrIncMemb, double agrIncMemb, double hhSize)
	{
		double constant = 433.867;
		double BETAnonAgrIncMemb = 0.337;
		double BETAagrIncMemb = 1.282;
		double BETAhhSize = -32.836;
		
		return constant+BETAnonAgrIncMemb*nonAgrIncMemb+BETAagrIncMemb*agrIncMemb+BETAhhSize*hhSize;
	}
	
	/**
	 * logical constrain on minimum food expenditures per HH member (minimum from survey data)
	 * @return
	 */
	public static double getMinFoodExp() 
	{
		return minFoodExp;
	}

	/**
	 * logical constrain on minimum non-food expenditures per HH member (minimum from survey data)
	 * @return
	 */
	public static double getMinNonFoodExp() 
	{
		return minNonFoodExp;
	}
	
	/**
	 * Simple method for expenditures for non-livestock owning HH based on distributions of different selling types (expenditure data for those where missing)
	 * @param type
	 * @return
	 */
	public static double getSpendRate(SellingType type)
	{
		double mean;
		double stDev;
		
		if(type==SellingType.survivor)
		{
			mean= 0.650384;
			stDev=0.330084;
		}
		else if(type==SellingType.spender)
		{
			mean= 0.85625;
			stDev= 0.22606415;
		}
		else if(type==SellingType.saver)
		{
			mean= 0.321;
			stDev= 0.146927798;
		}
		else if(type==SellingType.satisficer)
		{
			mean= 0.52375;
			stDev= 0.202339001;
		}
		else
			throw new IllegalArgumentException ("no or wrong selling type in gewtLuxSpendRate"); 

		RandomHelper.createNormal(mean, stDev);
		double randNr = RandomHelper.getNormal().nextDouble();
		if(randNr<0)
			randNr=0;
		
		return randNr;
	}
}
