package dataClasses;

import repast.simphony.random.RandomHelper;

/**
 * Returns the costs for a scenario with input costs (not used in the current paper)
 * @author karma
 *
 */
public class InputCostsCalculator 

{
	private static final double CONSTANT = 30.823;
	private static final double NON_AGRI_INCOM_COEFF = 0.037;
	private static final double TLU_COEFF = -8.283;
		
	/**
	 * Calculates input costs per TLU per month including fodder and veterany items. Logarithmic transformation.
	 * @param nonAgriIncomePerMemb
	 * @param TLU
	 * @return
	 */
	public static double getInputCostRegressionForAll_elacticity(double nonAgriIncomePerMemb, double TLU)
	{
		double constant = -1.51;
		double nonAgriIncCoef = 0.835;
		double tluCoef = -0.909;
		
		
		double logTlu = Math.log(TLU);
		double logInc = Math.log(nonAgriIncomePerMemb);
		
		double logY = (constant+(nonAgriIncCoef*logInc)+(logTlu*tluCoef));
		
		return Math.exp(logY);	
	}
	
	/**
	 * Calculates the monthly investment (costs) in supplementary fodder made per TLU
	 * @param nonAgriInc
	 * @param TLU
	 * @return costs per TLU
	 */
	public static double getFdrCostsRegression(double nonAgriInc, double TLU)
	{
		double constant = 10.792;
		double nonAgriIncCoef = 0.007;
		double tluCoeff = -0.843;
		
		double regression = constant+nonAgriIncCoef*nonAgriInc+tluCoeff*TLU;
		
		if(regression<0)
		{
			regression=0;
		}
		
		return regression;
	}
	
	
//----------Depreciated or unused methods------------------------------
	
	public static double getInputCostRegressionForAll(double nonAgriIncomePerMemb, double TLU)
	{
		double constant = 34.215;
		double nonAgriIncCoef = 0.015;
		double tluCoef = -17.384;
		
		double logTlu = Math.log(TLU);
		
		return constant+nonAgriIncCoef*nonAgriIncomePerMemb+logTlu*tluCoef;	
	}
	
	public static double getInputCostsUnobserved()
	{
		RandomHelper.createNormal(23.8834, 20.04301);
		return RandomHelper.getNormal().nextDouble();
	}
	
	public static double  getInputCostsRegression(double nonAgriIncomePerMemb, double TLU)
	{		
		return InputCostsCalculator.CONSTANT+
			   InputCostsCalculator.NON_AGRI_INCOM_COEFF*nonAgriIncomePerMemb+
			   InputCostsCalculator.TLU_COEFF*TLU;
	}
}
