package dataClasses;

import java.io.IOException;
import java.util.ArrayList;

import repast.simphony.random.RandomHelper;
import ses.ReaderHelper;
import enums.SellingType;

/**
 * Returns % selling rates of animals per selling type. Either from input files derived from cluster analysis or based on the empirical distributions from the survey
 * (not used for the current paper)
 * @author karma
 *
 */
public class SellingRates 
{
	/**
	 * List of selling rates (animals sold over all animals kept during the last year) for the cluster of survivors (poorest HH)
	 */
	private static  ArrayList <Double> surviviorRate;
	/**
	 * List of selling rates (animals sold over all animals kept during the last year) for the cluster of spenders (frequent seller HH)
	 */
	private static ArrayList <Double> spenderRate;
		
	static
	{
		try 
		{
			surviviorRate = ReaderHelper.getRates("SaleRate_Survivor");
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		try 
		{
			spenderRate = ReaderHelper.getRates("SaleRate_Spender");
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Draws the yearly selling rate of Survivor and Spender HH from the empirical distributions (data files)
	 * @param type HH type
	 * @return the selling rate (% of animals to sell in the current year)
	 */
	public static double getSaleRateEmpDstr(SellingType type)
	 {
		ArrayList<Double> rates = null;
		if(type==SellingType.spender)
		{
			rates=spenderRate;
		}
		else if(type==SellingType.survivor)
		{
			rates=surviviorRate;
		}
		else
		{
			throw new IllegalArgumentException("wrong HH type");
		}
		 RandomHelper.createUniform(0,Math.max(0,rates.size()-1));
		 int rateInstance = RandomHelper.getUniform().nextIntFromTo(0, rates.size()-1);
		 return rates.get(rateInstance);
	 }
	
	/**
	 * Draws the yearly selling rate of Survivor and Spender HH from a Normal distribution with a mean and stdev 
	 * calculated from the empircial selling rates.
	 * @param type
	 * @return
	 */
	public static double getSaleRateNrmlDstr(SellingType type)
	{
		double mean;
		double stDev;
		
		if(type==SellingType.survivor)
		{
			mean= 0.2745;
			stDev=0.19617;
		}
		else if(type==SellingType.spender)
		{
			mean= 0.1825;
			stDev= 0.13026;
		}
		else
			throw new IllegalArgumentException ("no or wrong selling type in gewtLuxSpendRate"); 

		RandomHelper.createNormal(mean, stDev);
		double randNr = RandomHelper.getNormal().nextDouble();
		if(randNr<0)
			randNr=0;
		
		return randNr;
	}
}
