package demoZLCS;

import java.util.Iterator;
import java.util.Random;
import classifierLibrary.ClassifierSystem;
import classifierLibrary.Situation;
import classifierLibrary.StandardClassifier;

/**
 * <p>Bibliothek demoZLCS</p><p>Klasse ZLCS</p><p>Copyright by Klaus
 * Hufschlag 2004,2005,2006</p><p>This file is part of the Generic Classifier
 * Library. The Generic Classifier Library is free software; you can
 * redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. The Generic Classifier
 * Library is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with
 * the Generic Classifier Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.</p>
 * @author Klaus Hufschlag
 * @version 1.4
 */
public class ZLCS extends ClassifierSystem {
	private ZLCS previousMatchset=null;
	private ZLCS matchset = null;
	private ZLCS actionset = null;
	private ZLCS previousactions = null;
	private ZLCS beforePreviousActions =null;
	private double P_wild = 0.0;
	private double S_init = 20;
	private double beta = 0.2;
	private double gamma = 0.71;
	private double tau = 0.05;
	private double chi = 0.9;
	private double my = 0.002;
	private double rho = 0.25;
	private double phi = 0.5;
	private int numVictimsGA =5;

	/**
	 * Reading accessor-method for beta.
	 * @return content of beta (double)
	 */
	public double getBeta() {
		return beta;
	}

	/**
	 * Writing accessor-method for beta.
	 * @param beta Value to be set
	 */
	public void setBeta(double beta) {
		this.beta = beta;
	}

	/**
	 * Reading accessor-method for chi.
	 * @return content of chi (double)
	 */
	public double getChi() {
		return chi;
	}

	/**
	 * Writing accessor-method for chi.
	 * @param chi Value to be set
	 */
	public void setChi(double chi) {
		this.chi = chi;
	}

	/**
	 * Reading accessor-method for gamma.
	 * @return content of gamma (double)
	 */
	public double getGamma() {
		return gamma;
	}

	/**
	 * Writing accessor-method for gamma.
	 * @param gamma Value to be set
	 */
	public void setGamma(double gamma) {
		this.gamma = gamma;
	}

	/**
	 * Reading accessor-method for my.
	 * @return content of my (double)
	 */
	public double getMy() {
		return my;
	}

	/**
	 * Writing accessor-method for my.
	 * @param my Value to be set
	 */
	public void setMy(double my) {
		this.my = my;
	}

	/**
	 * Reading accessor-method for numVictimsGA.
	 * @return content of numVictimsGA (int)
	 */
	public int getNumVictimsGA() {
		return numVictimsGA;
	}

	/**
	 * Writing accessor-method for numVictimsGA.
	 * @param numVictimsGA Value to be set
	 */
	public void setNumVictimsGA(int numVictimsGA) {
		this.numVictimsGA = numVictimsGA;
	}

	/**
	 * Reading accessor-method for p_wild.
	 * @return content of p_wild (double)
	 */
	public double getP_wild() {
		return P_wild;
	}

	/**
	 * Writing accessor-method for p_wild.
	 * @param p_wild Value to be set
	 */
	public void setP_wild(double p_wild) {
		P_wild = p_wild;
	}

	/**
	 * Reading accessor-method for phi.
	 * @return content of phi (double)
	 */
	public double getPhi() {
		return phi;
	}

	/**
	 * Writing accessor-method for phi.
	 * @param phi Value to be set
	 */
	public void setPhi(double phi) {
		this.phi = phi;
	}

	/**
	 * Reading accessor-method for rho.
	 * @return content of rho (double)
	 */
	public double getRho() {
		return rho;
	}

	/**
	 * Writing accessor-method for rho.
	 * @param rho Value to be set
	 */
	public void setRho(double rho) {
		this.rho = rho;
	}

	/**
	 * Reading accessor-method for s_init.
	 * @return content of s_init (double)
	 */
	public double getS_init() {
		return S_init;
	}

	/**
	 * Writing accessor-method for s_init.
	 * @param s_init Value to be set
	 */
	public void setS_init(double s_init) {
		S_init = s_init;
	}

	/**
	 * Reading accessor-method for tau.
	 * @return content of tau (double)
	 */
	public double getTau() {
		return tau;
	}

	/**
	 * Writing accessor-method for tau.
	 * @param tau Value to be set
	 */
	public void setTau(double tau) {
		this.tau = tau;
	}

	/** Construktor creating an empty instance of ZLCS */
	public ZLCS() {
		super();
	}

	/**
	 * Construktor creating an instance of ZLCS, using a given sample Classifier
	 * for filling the instance up to a defined size. Classifiers can be just
	 * clones of the sample or their rules can be randomized.
	 * @param name Name of the ClassifierSystem
	 * @param sample Sample classifier for filling the instance
	 * @param size Number of classifiers for initial filling
	 * @param randomized If true, the classifiers rule components will be
	 *        randomized
	 */
	public ZLCS(String name, StandardClassifier sample, int size,
			boolean randomized) {
		super(name, sample, size, randomized);
	}

	/**
	 * Returns the match-set (before covering) to a given Situation as a new
	 * instance.
	 * @param situation Situation to match
	 * @return New instance of ZLCS containing just the matching classifiers
	 */
	public ZLCS getMatchSet(Situation situation) {
		return (ZLCS) filterByCondition(situation);
	}

	/**
	 * Returns the match-set (after covering) to a given Situation as a new
	 * instance.
	 * @param sit Situation to match
	 * @return New instance of ZLCS containing the matching and covered
	 *         classifiers
	 */
	public ZLCS match_n_cover(Situation sit) {
		boolean check = false;
		ZLCS match = null;
		while (!check) {
			match = getMatchSet(sit);
			if (match.size() == 0
					|| match.getTotal("getStrength") < phi
							* getAverage("getStrength")) {
				cover(sit);// System.out.println("covering..................");
			} else {
				check = true;
			}
		}
		//previousMatchset=matchset;
		matchset = match;
		return match;
	}

	/**
	 * Returns the action-set matchin a given action as a new instance.
	 * @param action Action to match
	 * @return New instance of ZLCS containing the classifiers with matching
	 *         actions.
	 */
	public ZLCS getActionSet(Situation action) {
		actionset = (ZLCS) matchset.filterbyaction(action);
		return actionset;
	}

	/**
	 * ZLCS-Choice of an action matching a given condition
	 * @param condition Condition to match.
	 * @return Action chosen
	 */
	public Situation getActionChoice(Situation condition) {
		Situation action = match_n_cover(condition).chooseClassifier()
				.getRule().getAction();
		getActionSet(action);
		return action;
	}

	/**
	 * Roulettewheel-choice of a StandardClassifier, using the classifiers'
	 * strength for determining probabilities.
	 * @return StandardClassifier chosen
	 */
	public StandardClassifier chooseClassifier() {
		return (StandardClassifier) this.RoulettewheelChoice("getStrength",
				false);
	}

	/**
	 * Covering: Generation of a new StandardClassifier matching a given
	 * Situation.
	 * @param sit Situation to match
	 * @return New classifier
	 */
	public StandardClassifier cover(Situation sit) {
		StandardClassifier victim = (StandardClassifier) RoulettewheelChoice(
				"getStrength", true);
		StandardClassifier co = null;
		if (getSample() == null) {
			return null;
		}
		// Create matching, but only similar condition.
		Situation newCond = (Situation) sit.clone();
		newCond.setLanguage(getSample().getRule().getCondition().getLanguage());
		// Set Language of newCOnd
		Situation mutation = newCond.mutation(getMy()); // mutieren...
		while (mutation.compareSubsumtive(sit) < sit.size()) {
			mutation = newCond.mutation(getMy());
		}
		// Mutate until a mutation matches the condition.
		co = (StandardClassifier) getSample().clone();
		co.getRule().setCondition(mutation);
		co.getRule().getAction().randomizeSituation();
		co.setStrength(getAverage("getStrength"));
		remove(victim);
		add(co);
		return co;
	}

	/**
	 * Genetic Algorithm, only performed with a probability of <i>Rho </i>
	 */
	public void GA() {
		Random r = new Random();
		r.setSeed(4);
		if (r.nextDouble() < getRho()) {
			ClassifierSystem pool = (ClassifierSystem) this.clone();
			/* Selection */ClassifierSystem victims = new ClassifierSystem();
			for (int lauf = 1; lauf <= getNumVictimsGA(); lauf++) {
				StandardClassifier victim = (StandardClassifier) pool
						.RoulettewheelChoice("getStrength", true);
				pool.remove(victim);
				victims.add(victim);
			}
			ClassifierSystem offsprings = new ClassifierSystem();
			for (int lauf = 1; lauf <= getNumVictimsGA(); lauf++) {
				StandardClassifier offspring = (StandardClassifier) pool
						.RoulettewheelChoice("getStrength", false);
				pool.remove(offspring);
				offspring.setStrength((offspring.getStrength() / 2));
				offsprings.add((StandardClassifier) offspring.clone());
			}
			/* Crossover */
			if (r.nextDouble() < chi) {
				offsprings = offsprings.crossover();
				/* Mutation */
			}
			offsprings = offsprings.mutation(my);
			/* "Include" results into CS */remove(victims);
			add(offsprings);
		}
	}

	/**
	 * Processing of a reward
	 * @param reward Value of the reward to process.
	 */
	public void processReward(double reward) {
		double strength = 0;
		double bucket = 0;
		/*
		 * Stepwise implementation for better understandability:
		 */
		/* 1. Collect fees. */
		StandardClassifier c = null;
		Iterator i = actionset.iterator();
		while (i.hasNext()) {
			c = (StandardClassifier) i.next();
			strength = c.getStrength();
			bucket += strength * beta;
		}
		/* 2. Reward actions of previous period. */
		if (previousactions != null) {
			i = previousactions.iterator();
			bucket = gamma * bucket / previousactions.size();
			while (i.hasNext()) {
				c = (StandardClassifier) i.next();
				strength = c.getStrength();
				c.setStrength(strength + bucket);
			}
		}
		/* 3. Collect tax from Classifiers not drawn. */
		ClassifierSystem difference = new ClassifierSystem();
		difference.add(matchset);
		difference.remove(actionset);
		i = difference.iterator();
		while (i.hasNext()) {
			c = (StandardClassifier) i.next();
			strength = c.getStrength() * (1 - tau);
			c.setStrength(strength);
		}
		/* 4. Distribute reward. */
		double rew = reward / actionset.size();
		/* Iterator */i = actionset.iterator();
		while (i.hasNext()) {
			c = (StandardClassifier) i.next();
			c.setStrength(strength * (1 - beta) + rew * beta);
			if (c.getStrength() < 0)
				c.setStrength(0);
		}
		/* 5. Remember actual actionset for next round as the previous one */
		previousactions = actionset;
		actionset = null;
	}
	
	public ZLCS returnActionSet()
	{
		return actionset;
	}
}