package ses

import java.util.ArrayList
import java.util.Calendar
import java.util.List
import net.simplace.simulation.FWSimEngine
import net.simplace.simulation.FWSimSimulation
import net.simplace.simulation.io.logging.Logger
import net.simplace.simulation.io.logging.Logger.LOGLEVEL
import net.simplace.simulation.io.output.FWSimOutputManager
import net.simplace.simulation.model.FWSimModel
import net.simplace.simulation.util.FWSimVariable
import net.simplace.simulation.model.scala.FWSimVarConversions
import scala.collection.mutable.Queue
import scala.math.max
import net.simplace.simulation.model.scala.FiniteQueue._

class BiomassModelS(tSolutionName: String) extends FWSimVarConversions {
	
	private var iSimulations: List[FWSimSimulation] = null
	private var iModel: FWSimModel = null
	private var iFwSimSimulation: FWSimSimulation = null

	private var runCount = 0

	private var Offtake: FWSimVariable[Double] = null
	private var GFD: FWSimVariable[Double] = null

	private var LAI: FWSimVariable[Double] = null
	private var LAI_sen: FWSimVariable[Double] = null
	private var W_sh: FWSimVariable[Double] = null
	private var W_sen: FWSimVariable[Double] = null
	private var W_res: FWSimVariable[Double] = null
	private var W_rt: FWSimVariable[Double] = null
	private var BA: FWSimVariable[Double] = null
	private var dormCount: FWSimVariable[Integer] = null
	private var dormRDR: FWSimVariable[Double] = null
	private var f_T: FWSimVariable[Double] = null
	private var f_wt: FWSimVariable[Double] = null
	private var Tsum: FWSimVariable[Double] = null
	private var Temp: FWSimVariable[Double] = null
	private var Prec: FWSimVariable[Double] = null
	private var DayL: FWSimVariable[Double] = null
	private var dLAI: FWSimVariable[Double] = null
	private var dLAI_d: FWSimVariable[Double] = null
	private var dLAI_gr: FWSimVariable[Double] = null
	private var dLAI_sen: FWSimVariable[Double] = null
	private var dLAI_send: FWSimVariable[Double] = null
	private var dLAI_sengr: FWSimVariable[Double] = null
	private var dW_sh: FWSimVariable[Double] = null
	private var dW_shd: FWSimVariable[Double] = null
	private var dW_shgr: FWSimVariable[Double] = null
	private var dW_sen: FWSimVariable[Double] = null
	private var dW_send: FWSimVariable[Double] = null
	private var dW_sengr: FWSimVariable[Double] = null
	private var dW_res: FWSimVariable[Double] = null
	private var dW_rt: FWSimVariable[Double] = null
	private var GrGrowth: FWSimVariable[Double] = null
	
	// From SlimWater
	private var WA: FWSimVariable[Double] = null
	private var WAC: FWSimVariable[Double] = null
	private var Tran: FWSimVariable[Double] = null
	private var Evap: FWSimVariable[Double] = null
	private var Runoff: FWSimVariable[Double] = null
	
	//From STMP/SnowCover
	private var SoilSurfaceTemp: FWSimVariable[Double] = null
	private var STMP: FWSimVariable[Array[Double]] = null
	
	private var LayerThickness: FWSimVariable[Double] = null
	
	private val prodAboveGround = Queue[Double]()
	private val prodRoot = Queue[Double]()
	private val sumPrec = Queue[Double]()
	private val sumEvapTran = Queue[Double]()
	private var prevAG = 0d
	private var prevRt = 0d
	
	/* These variables are accessable using automatic Scala getters */
	
	/* Water use efficiencies
	 * Pr = precipitation
	 * ET = Evapotranspiration
	 * 1 = Aboveground
	 * 2 = Total: Root + Aboveground
	 */
	protected var WUE_Pr_A = 0d
	protected var WUE_Pr_T = 0d
	protected var WUE_ET_A = 0d
	protected var WUE_ET_T = 0d
	protected var RSratio = 0d
	protected var totalDays = 0
	
	/* Values for SES model */
	// Monthly green growth
	protected var GrGrowthMonth = 0d
	protected var actOfftake = 0d
	protected var actualGFD = 0d
	protected var actualIntMult = 0d
	

	// Constructor code
	Logger.setLogLevel(LOGLEVEL.INFO)
	Logger.INFO("Starting SIMPLACE", this)

	//val workSpFolder = "D:/Documents/Programming/SIMPLACE/simplacemod/linrange/"
	//val tSolutionName = workSpFolder + "simulation/conf/slim/LinrangeSlim.sol.xml"
	val tProjectController = FWSimEngine.prepareProject(null, tSolutionName)
	FWSimOutputManager.getInstance.runOutputWriters
	val tFwSimProject = tProjectController.getProjectMap.values.iterator.next

	iSimulations = tFwSimProject.init
	FWSimOutputManager.getInstance.clearSimulations
	

	def initRun {

		iFwSimSimulation = iSimulations.get(0).clone(runCount)
		val tActualSimulations = new ArrayList[FWSimSimulation]
		tActualSimulations.add(iFwSimSimulation)
		FWSimOutputManager.getInstance.registerSimulations(tActualSimulations)
		iFwSimSimulation.init
		iModel = iFwSimSimulation.getModel

		getSimVariables

		runCount = runCount + 1
	}

	private def getSimVariables {
		// Inputs
		Offtake = iModel.getVarMap.getSimVariable("Linrange.iOfftake")
		GFD = iModel.getVarMap.getSimVariable("Linrange.iGFD")

		// States Out
		LAI = iModel.getVarMap.getSimVariable("Linrange.LAI")
		LAI_sen = iModel.getVarMap.getSimVariable("Linrange.LAI_sen")
		W_sh = iModel.getVarMap.getSimVariable("Linrange.W_sh")
		W_sen = iModel.getVarMap.getSimVariable("Linrange.W_sen")
		W_res = iModel.getVarMap.getSimVariable("Linrange.W_res")
		W_rt = iModel.getVarMap.getSimVariable("SlimRoots.TotalRootBiomass")
		BA = iModel.getVarMap.getSimVariable("Linrange.BA")

		Offtake = iModel.getVarMap.getSimVariable("Linrange.iOfftake")
		dormCount = iModel.getVarMap.getSimVariable("Linrange.DormCount")
		dormRDR = iModel.getVarMap.getSimVariable("Linrange.DormRDR")
		f_T = iModel.getVarMap.getSimVariable("Linrange.f_T")
		f_wt = iModel.getVarMap.getSimVariable("LintulWaterStress.TRANRF")
		Tsum = iModel.getVarMap.getSimVariable("Linrange.Tsum")

		Temp = iModel.getVarMap.getSimVariable("weather_missingtransform.AirTemperatureMean")
		Prec = iModel.getVarMap.getSimVariable("weather_missingtransform.Rain")
		DayL = iModel.getVarMap.getSimVariable("dl.DayLength")

		// Rates
		dLAI = iModel.getVarMap.getSimVariable("Linrange.dLAI")
		dLAI_d = iModel.getVarMap.getSimVariable("Linrange.dLAI_d")
		dLAI_gr = iModel.getVarMap.getSimVariable("Linrange.dLAI_gr")
		dLAI_sen = iModel.getVarMap.getSimVariable("Linrange.dLAI_sen")
		dLAI_send = iModel.getVarMap.getSimVariable("Linrange.dLAI_send")
		dLAI_sengr = iModel.getVarMap.getSimVariable("Linrange.dLAI_sengr")
		dW_sh = iModel.getVarMap.getSimVariable("Linrange.dW_sh")
		dW_shd = iModel.getVarMap.getSimVariable("Linrange.dW_shd")
		dW_shgr = iModel.getVarMap.getSimVariable("Linrange.dW_shgr")
		dW_sen = iModel.getVarMap.getSimVariable("Linrange.dW_sen")
		dW_send = iModel.getVarMap.getSimVariable("Linrange.dW_send")
		dW_sengr = iModel.getVarMap.getSimVariable("Linrange.dW_sengr")

		dW_res = iModel.getVarMap.getSimVariable("Linrange.dW_res")
		dW_rt = iModel.getVarMap.getSimVariable("Linrange.dW_rt")
		
		GrGrowth = iModel.getVarMap().getSimVariable("Linrange.GrGrowth")
		
		WA = iModel.getVarMap.getSimVariable("SlimWater.TotalWaterContent")
		WAC = iModel.getVarMap.getSimVariable("SlimWater.TotalCropAvailWaterContent")
		Tran = iModel.getVarMap.getSimVariable("SlimWater.Transpiration")
		Evap = iModel.getVarMap.getSimVariable("SlimWater.Evaporation")
		Runoff = iModel.getVarMap.getSimVariable("SlimWater.WFAST")
		
		SoilSurfaceTemp = iModel.getVarMap.getSimVariable("SnowCoverCalculator.SoilSurfaceTemperature")
		STMP = iModel.getVarMap.getSimVariable("STMPsimCalculator.SoilTempArray")
		LayerThickness = iModel.getVarMap.getSimVariable("slim.THICKL")
	}

	/**
	 * offtake in g/m2
	 */
	def step(offtake: Double, gfd: Double) = {
		stepTime(Calendar.DAY_OF_MONTH, 1, offtake, gfd)
	}

	/**
	 * offtake in g/m2
	 */
	def stepTime(StepSize: Int, Steps: Int, offtake: Double, gfd: Double) = {

		// Write values to SIMPLACE varmap
		// Offtake per day
		Offtake.setValue(offtake)
		GFD.setValue(gfd)
		
		var dW_shgr_sum = 0d
		var dW_sengr_sum = 0d
		GrGrowthMonth = 0d

		val tCurrentCalendar = iModel.getVarMap.getCurrentCalendar
		var daysStepped = 0
		var stepped = 0
		var cont = false
		var tTimeStep = 0
		do {
			tTimeStep = tCurrentCalendar.get(StepSize)
			
			if (!tCurrentCalendar.equals(iModel.getEndDate)) {
				cont = iModel.step
			} else {
				Logger.INFO("No data for next FWSimModel step", this)
				cont = false
			}
			daysStepped += 1
			
			if (tTimeStep != tCurrentCalendar.get(StepSize)) {
				stepped += 1
			}
			
			{ // Calculate net productivity
				val period = 61
				val AG = W_sh.getValue + W_sen.getValue
				val Rt = W_rt.getValue
				prodAboveGround.appendMax(AG - prevAG, period)
				prodRoot.appendMax(Rt - prevRt, period)
				sumPrec.appendMax(Prec, period)
				sumEvapTran.appendMax(Tran.getValue + Evap.getValue, period)
				prevAG = AG
				prevRt = Rt
			}
			{ // Sum daily green growth and grazing values
				GrGrowthMonth += GrGrowth.getValue();
				dW_shgr_sum += dW_shgr.getValue();
				dW_sengr_sum += dW_sengr.getValue();
			}
		} while ((stepped < Steps) && cont)
		totalDays += daysStepped
		
		{ // Values for SES model
			actOfftake = dW_shgr_sum + dW_sengr_sum;
			actualGFD = dW_shgr_sum / actOfftake;
			if(actualGFD.isNaN()) {
				actualGFD=0d;
			}		
			actualIntMult = actOfftake/offtake;
		}
		
		// Check for uneven month number, i.e. Feb (1), April (3), etc.
		if ((totalDays>365) && (tCurrentCalendar.get(Calendar.MONTH) % 2 == 1)) {
			if (sumPrec.sum == 0) {
				WUE_Pr_A = 0
				WUE_Pr_T = 0
			} else {
				WUE_Pr_A = max(prodAboveGround.sum / sumPrec.sum, 0)
				WUE_Pr_T = max((prodAboveGround.sum + prodRoot.sum) / sumPrec.sum, 0)
			}
			WUE_ET_A = max(prodAboveGround.sum / sumEvapTran.sum, 0)
			WUE_ET_T = max((prodAboveGround.sum + prodRoot.sum) / sumEvapTran.sum, 0)
		}
		RSratio = (W_rt.getValue / (W_sh.getValue + W_sen.getValue))

		cont
	}
	
	def finishRun {
		// Finalise a model run of simplace
		iFwSimSimulation.finish
		FWSimOutputManager.getInstance.clearSimulations
	}
	

	def finish {
		// Finalise SIMPLACE
		FWSimEngine.getInstance.shutDown
		Logger.INFO("Shut down SIMPLACE", this)
	}

	def getLAI_tot: Double = {
		LAI.getValue + LAI_sen.getValue
	}

	def getLAI: Double = {
		LAI
	}

	def getLAI_sen: Double = {
		LAI_sen
	}

	def getW_tot: Double = {
		W_sh.getValue + W_sen.getValue
	}

	def getW_sh: Double = {
		W_sh
	}

	def getW_sen: Double = {
		W_sen
	}

	def getW_res: Double = {
		W_res
	}

	def getW_rt: Double = {
		W_rt
	}

	def getBA: Double = {
		BA
	}

	def getWA: Double = {
		WA
	}

	def getWAC: Double = {
		WAC
	}
	
	def getTran: Double = {
		Tran
	}

	def getEvap: Double = {
		Evap
	}
	
	def getRunoff: Double = {
		Runoff
	}
	
	def getDormCount: Double = {
		dormCount.getValue.doubleValue
	}

	def getDormRDR: Double = {
		dormRDR
	}

	def getF_T: Double = {
		f_T
	}

	def getF_wt: Double = {
		f_wt
	}

	def getTsum: Double = {
		Tsum
	}

	def getTemp: Double = {
		Temp
	}

	def getPrec: Double = {
		Prec
	}

	def getDayL: Double = {
		DayL
	}

	def getdLAI: Double = {
		dLAI
	}

	def getdLAI_d: Double = {
		dLAI_d
	}

	def getdLAI_gr: Double = {
		dLAI_gr
	}

	def getdLAI_sen: Double = {
		dLAI_sen
	}

	def getdLAI_send: Double = {
		dLAI_send
	}

	def getdLAI_sengr: Double = {
		dLAI_sengr
	}

	def getdW_sh: Double = {
		dW_sh
	}

	def getdW_shd: Double = {
		dW_shd
	}

	def getdW_shgr: Double = {
		dW_shgr
	}

	def getdW_sen: Double = {
		dW_sen
	}

	def getdW_send: Double = {
		dW_send
	}

	def getdW_sengr: Double = {
		dW_sengr
	}

	def getdW_res: Double = {
		dW_res
	}

	def getdW_rt: Double = {
		dW_rt
	}
	
	def getSumPrec: Double = {
		sumPrec.sum
	}
	


}
