package ses;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jct.util.Gini;
import jct.util.SortedEnumeration;
import classifierLibrary.StandardSituation;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.essentials.RepastEssentials;
import repast.simphony.parameter.Parameters;
import agents.EntryAble;
import agents.ProducerHH;
import agents.SimpleHH;
import animals.Animal;

public class GlobSpaceImpl implements GlobSpace
{
	//objects
	private ArrayList<EntryAble> allOtherHH;
	private ArrayList<ProducerHH> allProducerHH;
	private EcoSpace theRangeland;
	private SocialSpace theCommunity;
	
	//starting month
	private int currentMonth=7;
	
	//Repast output variables
	private int exitCounter=0;
	private boolean wasCooperation=false;
	private boolean wasCollapse=false;
	private double nrSoldAnimals;
	private double forgoneSavings;
	private double sumOfProfits;
	
	/**
	 * The warm up time for the BiomassSimulator in months. No other steps are performed during this time.
	 */
	private int initTime=59;
	
	public GlobSpaceImpl(ArrayList<EntryAble> listOfSimpleHH, ArrayList<ProducerHH> listOfProudecerHH, Context theContext)
	{
		allOtherHH = listOfSimpleHH;
		allProducerHH = listOfProudecerHH;
		theRangeland = new EcoSpace(theContext);
		theCommunity = new SocialSpace();
				
		theContext.add(theRangeland);
		theContext.add(theCommunity);
	}
	
	/**
	 * Global steppping method
	 */
	public void stepGlobal()
	{
		updateMonth();
		if(RepastEssentials.GetTickCount()>=initTime)
		{
			stepAnimalAgeing();
			stepBiomassGrowth();		
			stepLiveweightgain();
			stepHHbalance();
			if(currentMonth==7)
			{
				stepYearlyLwg();
				stepBirthsBranding();
				stepDeaths();			
			}				
			stepIndiSelling();
			stepNormSelling();
			stepCooperation();	
			stepExit();	
			stepEntry();
			checkWasCooperation();//only output method
			if(currentMonth==12)
				stepResetSavings();
		}
		else
			theRangeland.init(currentMonth);
	}
	
	/**
	 * updates the current month of the year
	 */
	private void updateMonth()
	{
		if(this.currentMonth<=11)
		{
			this.currentMonth=this.currentMonth+1;
		}
		else
		{
			this.currentMonth=1;
		}	
	}
	
	/**
	 * Ages all Animals by one month
	 */
	public void stepAnimalAgeing()
	{
		for(ProducerHH aProducer:allProducerHH)
		{
			aProducer.ageYourAnimals();
		}
	}
	
	/**
	 * Steps the biomass growth during the last month. That is, the BiomassSimulator steps the linrange model
	 * for 31 days and adapts the biomass variables thereafter.
	 */
	public void stepBiomassGrowth()
	{
		theRangeland.stepMonthlyBiomass(currentMonth, getAllAnimals());
	}
	
	/**
	 * Gets the live-weight gain resulted from resource biomass consumption and informs
	 * all ProducerHH with this value. Also, the Rangeland holds this value in an Array over the year.
	 */
	public void stepLiveweightgain()
	{	
		double lwg = theRangeland.getLiveweightgain();
		
		for(ProducerHH aProducer: allProducerHH)
		{
			aProducer.informLwg(lwg);
		}
		
		theRangeland.storeMonhtlyLwg(lwg);
	}
		
	/**
	 * Steps the HH-balance of all HH
	 */
	public void stepHHbalance()
	{
		for(ProducerHH aProducer: allProducerHH)
		{
			aProducer.hhBalanceSimple();	
		}
		for(SimpleHH anOtherHH: allOtherHH)
		{
			anOtherHH.hhBalanceTypes();
		}
	}
	
	/**
	 * Steps the yearly live-weight gain of cattle
	 */
	public void stepYearlyLwg()
	{
		double lwgLastYear= theRangeland.calcYearlyLwg();
		theRangeland.setLwgLastYear(lwgLastYear);
	}
	
	/**
	 * Steps yearly net reproduction by using the branding rate
	 */
	public void stepBirthsBranding()
	{	
		double lwgLastYear= theRangeland.getYearlyLwg();
		for(ProducerHH aProducer: allProducerHH)
		{
			aProducer.birthYourAnimalsBrand(lwgLastYear);			
		}
	}
	
	/**
	 * Steps yearly mortality 
	 */
	public void stepDeaths()
	{
		double lwgLastYear= theRangeland.getYearlyLwg();
		for(ProducerHH aProducer: allProducerHH)
		{
			aProducer.dieYourAnimals(lwgLastYear);
		}
		theRangeland.resetYearlyLwg();
		
		if(LiveWeightGain.getMortality(lwgLastYear)>=1)
			wasCollapse=true;
	}
	
	/**
	 * Steps the individual selling decision of ProducerHH.
	 */
	public void stepIndiSelling()
	{		
		ArrayList<Animal> soldAnimals = new ArrayList<Animal>();
		for(ProducerHH aProducer:allProducerHH)
		{	
			soldAnimals.addAll(aProducer.sellIndividual());
		}		
	}
	
	/**
	 * steps sales triggerd by the norm
	 */
	public void stepNormSelling() 
	{	
		ArrayList<Animal>soldAnimals= new ArrayList<Animal>();
		for(ProducerHH aProducer:allProducerHH)
		{		
			soldAnimals.addAll(aProducer.sellNorm());
		}		
	}

	/**
	 * Steps the calculation of group variables needed for the calculation of norm compliance (group reputation, severity, vividness)
	 */
	public void stepCooperation()
	{
		theCommunity.stepCollectiveAction(getGiniHerdSizes(), allProducerHH);
	}
	
	/**
	 * Checks if a Producer HH has to exit. A other HH is created in 
	 * the ProducerHH and added to the list. The former ProducerHH is then removed from the list
	 * in GlobSpace.
	 */
	public void stepExit() 
	{
		ArrayList<ProducerHH> exits = new ArrayList<ProducerHH>();
		ArrayList<EntryAble> entries = new ArrayList<EntryAble>();
		for(ProducerHH aProducer: allProducerHH)
		{
			if(aProducer.isExit())
			{
				exits.add(aProducer);
				entries.add(aProducer.exit());		
			}
		}
		allProducerHH.removeAll(exits);
		allOtherHH.addAll(entries);
	}
	
	/**
	 * Checks if non-livestock owning HH want to enter and adds them the list of producing HH.
	 * Non-owners are then removed from the list of other HH. The creation of new HH is
	 * done within the agent object.
	 */
	public void stepEntry() 
	{
		ArrayList<EntryAble> exits = new ArrayList<EntryAble>();
		ArrayList<ProducerHH> entries = new ArrayList<ProducerHH>();
		for(EntryAble otherHH: allOtherHH)
		{
			if(otherHH.isEntry())
			{
				exits.add(otherHH);
				entries.add(otherHH.entry());	
				incrExit();
				forgoneSavings+=4000;
			}
		}
		allOtherHH.removeAll(exits);
		allProducerHH.addAll(entries);	
	}
	
	/**
	 * Sets accumulated savings to zero in order to cover non-observed expenses during the year.
	 */
	public void stepResetSavings()
	{
		for(ProducerHH aProducer: allProducerHH)
		{
			aProducer.resetAccSavings();
		}
		for(SimpleHH anOtherHH: allOtherHH)
		{
			anOtherHH.resetAccSavings();
		}
	}
	

////////////helper methods/////////////////////////
	
	/**
	 * gets a list of all cattle
	 * @return
	 */
	private ArrayList <Animal> getAllAnimals()
	{
		ArrayList<Animal> allAnimals = new ArrayList<Animal>();
		for(ProducerHH aProducer: allProducerHH)
		{
			allAnimals.addAll(aProducer.getAnimals());
		}		
		return allAnimals;
	}

	/**
	 * 
	 * @return the gini coefficient of herd sizes. Does only iterate over ProducerHH
	 */
	public double getGiniHerdSizes()
	{
		SortedEnumeration<Double> list = new SortedEnumeration<Double>();		
		ArrayList<Double> herdSizes = new ArrayList<Double>();
		for(ProducerHH aProducer: allProducerHH)
		{
			herdSizes.add(aProducer.getHerdSize());
		}
		for(Double aSize:herdSizes)
		{
			list.insert(aSize);
		}
		double gini = Gini.compute(list, false);
		if(Double.isNaN(gini))
		{
			gini=0;
		}
		return gini;
	}
	
	/**
	 * 
	 * @return Gets the gini over agricultural profits
	 */
	public double getGiniProfits()
	{
		SortedEnumeration<Double> list = new SortedEnumeration<Double>();		
		ArrayList<Double> profits = new ArrayList<Double>();
		for(ProducerHH aProducer: allProducerHH)
		{
			double prof = aProducer.getAgriRev();
			if(prof<0)
				prof=0;
			profits.add(prof);
		}
		for(Double aProfit:profits)
		{
			list.insert(aProfit);
		}
		double gini = Gini.compute(list, true);
		if(Double.isNaN(gini))
		{
			gini=0;
		}
		return gini;
	}
	
	
///////////////////output variables///////////////////////////////////////
	/**
	 * Gets the sum of all profits accumulated over the  run
	 * @return
	 */
	public double getSumOfProfits()
	{		 
		for(ProducerHH aProducer: allProducerHH)
		{
			sumOfProfits+=aProducer.getAgriRev();
		}

		return sumOfProfits;
	}
	
	/**
	 * Gets the % of producers who have only one animal
	 * @return
	 */
	public double getPercProducerWithOneAnimal()
	{
		double sum=0;
		for(ProducerHH aProducer: allProducerHH)
		{
			if(aProducer.getHerdSize()<2)
			{
				sum++;
			}
		}		
		return sum/allProducerHH.size();
	}
	
	/**
	 * 
	 * @return the current stocking rate in ha/TLU
	 */
	public double getStockingRate()
	{		
		return theRangeland.getStockingRate(getAllAnimals());
	}
	
	/**
	 * Gets all forage consumed during the lat month
	 * @return
	 */
	public double getOfftake()
	{
		return theRangeland.getOfftake(getAllAnimals());
	}

	/**
	 * Gets last years mortality
	 * @return
	 */
	public double getMort() 
	{	double lwgLastYear= theRangeland.getLastYearLwg();
		double mort = LiveWeightGain.getMortality(lwgLastYear);
		return mort;
	}

	/**
	 * Gets last years branding rate
	 * @return
	 */
	public double getBrand() 
	{
		double lwgLastYear= theRangeland.calcYearlyLwg();
		double brand = LiveWeightGain.getBrandRate(lwgLastYear);
		return brand;
	}
	
	/**
	 * increments the count of exits of producers
	 */
	public void incrExit()
	{
		exitCounter++;
	}
	
	/**
	 * Gets all exits of HH over the run
	 * @return
	 */
	public int getExitNr()
	{
		return exitCounter;
	}
	
	/**
	 * Gets the current total value of the herd on the rangeland
	 * @return
	 */
	public double getValueOfTotalHerd()
	{
		double val=0;
		for(ProducerHH aProducer: allProducerHH)
		{
			val+=aProducer.getValueOfHerd();
		}
		return val;
	}
	
	/**
	 * Gets the mean value of the herd on the rangeland
	 * @return
	 */
	public double getMeanValueOfHerds()
	{
		double val=getValueOfTotalHerd();
		return val/allProducerHH.size();
	}

	/**
	 * Gets the standard deviation of the herd on the rangeland
	 * @return
	 */
	public double getStdevValueOfHerds()
	{
		double var=0;
		double mean =getMeanValueOfHerds();
        double temp = 0;
        for(ProducerHH aProducer: allProducerHH)
            temp += (mean-aProducer.getValueOfHerd())*(mean-aProducer.getValueOfHerd());
            
        var = temp/allProducerHH.size();
        
        return Math.sqrt(var);
	}
	
	/**
	 * Gets the standard deviation of herd sizes in the current time step
	 * @return
	 */
	public double getStdevHerdSizes()
	{
		double var=0;
		double mean = getMeanHerdSizes();
        double temp = 0;
        for(ProducerHH aProducer: allProducerHH)
            temp += (mean-aProducer.getHerdSize())*(mean-aProducer.getHerdSize());
            
        var = temp/allProducerHH.size();
        
        return Math.sqrt(var);
	}
	
	/**
	 * Gets the arithmetic mean of herd sizes
	 * @return
	 */
	public double getMeanHerdSizes()
	{
		double val=getAllAnimals().size();
		return val/allProducerHH.size();
	}

	/**
	 * Gets the number of animals 
	 * @return
	 */
	public double getNrSoldAnimals() {
		return nrSoldAnimals;
	}


	/**
	 * Gets the modus of herd sizes
	 * @return
	 */
	public double modeHerdSize()
	{
		ArrayList<Double> sizeClasses = new ArrayList<Double>();
		for(ProducerHH aProducer: allProducerHH)
		{
			sizeClasses.add(aProducer.getHerdSize());
		}	
		double aMode =  mode(sizeClasses);
		return aMode;
	}
	
	/**
	 * Helper method for the mode of herd sizes
	 * @param coll
	 * @return
	 */
	private static <T> T mode(List<? extends T> coll) 
	{
	        Map<T, Integer> seen = new HashMap<T, Integer>();
	        int max = 0;
	        List<T> maxElems = new ArrayList<T>();
	        for (T value : coll) {
	            if (seen.containsKey(value))
	                seen.put(value, seen.get(value) + 1);
	            else
	                seen.put(value, 1);
	            if (seen.get(value) > max) {
	                max = seen.get(value);
	                maxElems.clear();
	                maxElems.add(value);
	            } else if (seen.get(value) == max) {
	                maxElems.add(value);
	            }
	        }
	        return maxElems.get(0);
	  }
	
	/**
	 * Gets the median of herd sizes
	 * @return
	 */
	public  double medianHerdSizes()
	{
		ArrayList<Double> sizeClasses = new ArrayList<Double>();
		for(ProducerHH aProducer: allProducerHH)
		{
			sizeClasses.add(aProducer.getHerdSize());
		}
		if(!sizeClasses.isEmpty())
		{
			Collections.sort(sizeClasses);
			   return (sizeClasses.get(sizeClasses.size() / 2) + sizeClasses.get((sizeClasses.size() - 1) / 2)) / 2;
		}
		else
			return 0;	   
	}
	
	/**
	 * Gets all investment costs for bought animals
	 * @return
	 */
	public double getInvestmentCosts()
	{
		return forgoneSavings;
	}

	/**
	 * returns binary variable for cooperation
	 * @return
	 */
	public int getWasCooperation()
	{
		if(wasCooperation)
			return 1;
		else
			return 0;
	}
	
	/**
	 * returns binary variable for collapse
	 * @return
	 */
	public int getWasCollapse()
	{
		if(wasCollapse)
			return 1;
		else
			return 0;
	}
	
	/**
	 * finishes the run of rangeland model
	 */
	public void finishBiomass()
	{
		theRangeland.finishRunBiom();
		theRangeland=null;	
	}
	
	/**
	 * Repast output method for cooperation
	 */
	private void checkWasCooperation()
	{
		if(RepastEssentials.GetTickCount()<1200)
		{
			double cooperation = theCommunity.getGroupTrust();
			if(cooperation>=1)
			{
				wasCooperation=true;
			}
		}		
	}
	
//////returns parameters from paramenter file for repast output///////////////	
	public int getParameterCooperationIni()
	{
		Parameters params = RunEnvironment.getInstance().getParameters();
		double coopIniAttr = (double)params.getValue("Cooperativeness");
		return (int) coopIniAttr;
	}
	
	public int getParameterHHAttrIni()
	{
		Parameters params = RunEnvironment.getInstance().getParameters();
		double hhAttrIni = (double)params.getValue("HH_attributes");
		return (int) hhAttrIni;
	}
	
	public int getParameterHerdSizeIni()
	{
		Parameters params = RunEnvironment.getInstance().getParameters();
		double  herdsizeIni = (double)params.getValue("Ini_HerdSize");
		return (int) herdsizeIni;
	}
	
	public int getParameterMaxHerdSize()
	{
		Parameters params = RunEnvironment.getInstance().getParameters();
		double maxHerdSize = (Double)params.getValue("maxHerdSize");
		if(maxHerdSize==25)
			return 0;
		else if(maxHerdSize==20)
			return 1;
		else if (maxHerdSize==30)
			return 2;
		else return 9999;
	}
	
}
