package ses;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;

/**
 * Livestock model from Gross et al. 2006. We thank the authors for providing the code
 * @author karma
 *
 */
public class LiveWeightGain
{
	
	/**
	 * calculates monthly live-weight gain or loss in kg
	 * @param tsc
	 * @param gsc
	 * @param lastGrGrowth
	 * @param actGFD
	 * @return
	 */
	public static double  calculateLWG (double tsc, double gsc, double lastGrGrowth, double actGFD)
	{
		Parameters params = RunEnvironment.getInstance().getParameters();
		double lwgMax =(Double) params.getValue("LWG_6_lwgMax");
		double lwgMin= (Double) params.getValue("LWG_7_lwgMin");
		double lwgRange = lwgMax-lwgMin;
		double dietQuality=getDietQuality(tsc,gsc,lastGrGrowth, actGFD);
		double intakeMult=getIntakeMult(tsc);
		double lwg = (lwgRange*dietQuality*intakeMult)+lwgMin;
		
		Double lwgd = lwg;
		assert(!lwgd.isNaN()) : "live-weight gain is NAN in LWGCalculator!";	
		
		return lwg;
	}
		
	/**
	 * Gets the quality of the forage
	 * @param tsc
	 * @param gsc
	 * @param lastGrGrowth
	 * @param actGFD
	 * @return
	 */
	public static double getDietQuality(double tsc, double gsc, double lastGrGrowth, double actGFD)
	{
		assert(gsc!=0) : "gsc is zero in LWGCalculator";
		
		Parameters params = RunEnvironment.getInstance().getParameters();		
		double greenInDiet=getGreenInDiet(tsc,gsc);
		double qualMinValue =(Double) params.getValue("LWG_4_qualMinValue");
		double slopeGreenGr =(Double) params.getValue("LWG_5_slopeGreenGr");
		double dietQuality=greenInDiet*((qualMinValue+((slopeGreenGr*lastGrGrowth))/(gsc+lastGrGrowth+1)));
		//double dietQuality=actGFD*((qualMinValue+((slopeGreenGr*gsc))/(gsc+lastGrGrowth+1)));
		
		return dietQuality;
	}
	
	/***
	 * Gets the share of green fodder in the diet
	 * @param tsc
	 * @param gsc
	 * @return
	 */
	public static double getGreenInDiet(double tsc, double gsc)
	{
		Parameters params = RunEnvironment.getInstance().getParameters();	
		double selecCoeff=(Double) params.getValue("LWG_3_selecCoeff");
		double propGrn=gsc/tsc;
		double greenInDiet = (selecCoeff*propGrn)/(selecCoeff*propGrn+1-propGrn);
		
		return greenInDiet;
	}
	
	/**
	 * Gets the intake multiplier on total intake
	 * @param tsc
	 * @return
	 */
	public static double getIntakeMult(double tsc)
	{		
		Parameters params = RunEnvironment.getInstance().getParameters();		
		double intakeCoeff1 = (Double) params.getValue("LWG_1_intakeCoeff1");
		double intakeCoeff2 =(Double) params.getValue("LWG_2_intakeCoeff2");
		double intakeMult = Math.min((intakeCoeff1*(tsc/(tsc+intakeCoeff2))), 1);
		
		return intakeMult;
	}
	
	/**
	 * Gets yearly % mortality
	 * @param lwg
	 * @return
	 */
	public static double getMortality(double lwg)
	{
		double mort = (3.2+87*Math.exp(-0.03*(lwg+50)))/100;
		if(mort<0)
		{
			mort=0;
		}
		else if(mort>1)
		{
			mort=1;
		}
		return mort;
	}
	
	/**
	 * Gets yearly % net reproduction
	 * @param lwg
	 * @return
	 */
	public static double getBrandRate(double lwg)
	{
		double brandingRate = 15.6+0.488*lwg;
		if (brandingRate>80)
			brandingRate=80;	
		
		return (brandingRate/100);
	}
}
