package ses;

import java.util.ArrayList;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import animals.Animal;
import animals.Bull;
import animals.Cow;
import animals.Heifer;
/**
 * The MarketSpace calculates the revenue from livestock sales for each agent. Selling price for calves/heifers is fixed
 */
public class MarketSpace 
{
	/**
	 * minium selling price for adult cattle taken from survey data
	 */
	private double minSellPAlt=3000;
	/**
	 * fixed selling price for calves/heifers
	 */
	private double sellingPrHeifer=1500;
	
	/**
	 * Called by Livestock producers who sold animals.
	 * @param soldAnimalList all kind of cattle sold
	 * @return the revenue from sales
	 */
	public double getTotalRevenue(ArrayList<Animal> soldAnimalList)
	{	
		double totalRev = 0;
		if(!soldAnimalList.isEmpty())
		{
			for(Animal anAnimal:soldAnimalList)
			{
				if(anAnimal instanceof Bull)
				{
					double rev = getSellingPriceSigmoid(anAnimal.getBodyWeight());
					totalRev+=rev;				
				}
				else if(anAnimal instanceof Cow)
				{
					double rev = getSellingPriceSigmoid(anAnimal.getBodyWeight());
					totalRev+=rev;		
				}
				else if(anAnimal instanceof Heifer)
				{
					totalRev+=sellingPrHeifer;
				}			
			}			
		}
		return totalRev;	
	}
	
	private double getSellingPriceSigmoid(double lw)
	{
		double price=minSellPAlt;
		if(lw>300)
		{
			double lwAdd = lw-300;
			price = price+(2000*(1/(1+Math.exp((5-(lwAdd*((0.1))))))));
		}
		return price;
	}
}
