package ses;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import classifierLibrary.Language;
import classifierLibrary.Rule;
import classifierLibrary.Situation;
import classifierLibrary.StandardClassifier;
import classifierLibrary.StandardSituation;
import classifierLibrary.Symbol;
import classifierLibrary.TestSituation;
import dataClasses.SellingRates;
import demoZLCS.ZLCS;
import actions.Selling;
import actions.SellingImpl;
import agents.EntryAble;
import agents.OtherHHImpl;
import agents.ProducerHH;
import agents.ProducerHHImpl;
import agents.SimpleHH;
import agents.AbstractHH;
import repast.simphony.context.Context;
import repast.simphony.context.DefaultContext;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.essentials.RepastEssentials;
import repast.simphony.parameter.Parameters;

public class SesModel extends DefaultContext<Object> implements ContextBuilder<Object>
{
	private GlobSpace theGlobSpace;
	private Language langA;
	private Language langB;
	private Symbol firstSymbol;
	private Symbol secondSymbol;
	private static int  count=0;//static counter of instants 

	@Override
	public Context build(Context<Object> context) 
	{
		context.setId("SES");
		System.out.println("run = "+count);
		Parameters params = RunEnvironment.getInstance().getParameters();
		
		//creation of Symbols and Languages for the learning classifier system 
		//Copyright by * Klaus Hufschlag 2004,2005,2006. Used with permission of author.	 
		//(here only the GA is used)
		if(count>0)
		{
			this.clear();
			firstSymbol.removeGlobal();
			secondSymbol.removeGlobal();
			langA.remove();
			langB.remove();
			theGlobSpace=null;//important to delete reference: Memory lack in batch runs
		}
		
		firstSymbol = new Symbol("0");
		secondSymbol = new Symbol("1");
		langB = new Language("LangB");
		langA = new Language("LangA");
		langA.add(firstSymbol);
		langA.add(secondSymbol);
		langB.add(langA);
		
	//creation of HH agents and singleton objects	
		MarketSpace aMarket = new MarketSpace();
		context.add(aMarket);
		
		ArrayList<ProducerHH> allProducerHH = null;
		ArrayList<EntryAble> allOtherHH = null;
		
		allProducerHH = createHH("ProducerHHdata", context, true,aMarket);
		allOtherHH =	createHH("SimpleHHdata",context,false,aMarket);
		
		theGlobSpace = new GlobSpaceImpl(allOtherHH,allProducerHH,context);
		context.add(theGlobSpace);
		
		//stepping
		int endAt = (Integer) params.getValue("stopAt");
		
		ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
		ScheduleParameters globalStep = ScheduleParameters.createRepeating(1,1,1);
		ScheduleParameters stop = ScheduleParameters.createAtEnd(ScheduleParameters.LAST_PRIORITY);
		schedule.schedule(globalStep,this,"globalStep");
		RunEnvironment.getInstance().endAt(endAt);	
		schedule.schedule(stop, this, "endMethod");		
		count++;
		
		return context;
	}
	
	/**
	 * global stepping method
	 */
	public void globalStep()
	{
		theGlobSpace.stepGlobal();
	}
	
	public void endMethod()
	{
		theGlobSpace.finishBiomass();
	}
	
	/**
	 * creates the HH agents at model start by reading in csv files.
	 * @param fileName file name of data input csv file
	 * @param theContext the model context
	 * @param producer if HH agents owns livestock or not
	 * @return a list of HH agents
	 */
	private ArrayList createHH (String fileName, Context theContext, boolean producer, MarketSpace aMarket)
	{	
		//agent creation from a file
		ArrayList HHs = new ArrayList();
		double [][] partData = null;
		try 
		{
				partData = ReaderHelper.readAllData(fileName);
		} 
		catch (IOException e) 
		{
				System.out.println("Failed to read the CSV File");
				e.printStackTrace();
		}
		//nr of participants = rows in file (first data entry in first matrix row)
		int participantCount = (int) partData[0][0];//
		//nr of attributes = cols in file (second data entry in first matrix row)
		int attributesNr = (int) (partData[0][1]);

		//reading of participant data and creation of single participants
		for (int i=1; i<= participantCount; i++)
		{
			//new array for one Participant
			double [] specParData = ReaderHelper.extractSingleParticipantData(i, attributesNr, partData);
			//hash map creation
			HashMap<String, Double> parDMap = new HashMap<String, Double>();
			try {
					parDMap = ReaderHelper.createMap(specParData, attributesNr, fileName);
				} 
			catch (IOException e) 
				{
						System.out.println("failed to build attribute map for participant");
						e.printStackTrace();
				}
			//here only one SellingDecision per HH is generated and handed over in case of entry/exit
			Selling mySellingDecsion = new SellingImpl(getLCS(), getInitSituation(),getSecondSituation(),aMarket);
			theContext.add(mySellingDecsion);
			if(producer)
			{
				ProducerHH aProducer = new ProducerHHImpl(theContext, mySellingDecsion,theGlobSpace);
				aProducer.iniWithFileData(parDMap);
				aProducer.iniCooperator();
				HHs.add(aProducer);		
			}
			else
			{
				EntryAble anOtherHH = new OtherHHImpl(theContext,mySellingDecsion,theGlobSpace);
				anOtherHH.iniWithFileData(parDMap);
				anOtherHH.iniCooperator();
				HHs.add(anOtherHH);			
			}
		}				
		return HHs;	
	}
	
	
	/**
	 * creates a learning classifier system with a rule consisting out of an condition and an action
	 * @return classifier system object
	 */
	private ZLCS getLCS()
	{	
		TestSituation condSit = new TestSituation(langB,1);
		TestSituation actionSit = new TestSituation(langA,12);
		Rule aRule = new Rule(condSit,actionSit);
		
		StandardClassifier ourClassifier = new StandardClassifier(aRule,1);
		
		ZLCS aClassifierSystem = new ZLCS("testCS", ourClassifier, 50,true);
		
		return aClassifierSystem;
	}
	
	/**
	 * creates an inital situation for the condition part of the classifier
	 * @return
	 */
	private StandardSituation getInitSituation()
	{
		TestSituation newSituation = new TestSituation(langB,1);
		newSituation.setElement(0,firstSymbol);
		
		return newSituation;
	}
	
	/**
	 * creates a second Situation for the condition part of the classifier
	 * @return
	 */
	private StandardSituation getSecondSituation()
	{
		TestSituation newSituation = new TestSituation(langB,1);
		newSituation.setElement(0,secondSymbol);
		
		return newSituation;
	}
}
